/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.forceten.application.capabilities;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javax.swing.Action;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.semantic.Wrapper;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProviderSupport;
import it.tidalwave.netbeans.nodes.role.ActionProvider;

/***********************************************************************************************************************
 *
 * FIXME: once you figure out how to remove the dependency on Wrapper and Elmo, move this to OpenBlueSky.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultActionProviderProvider<T> extends CapabilitiesProviderSupport<T>
  {
    private final Map<String, Collection<? extends ActionProvider>> capabilityMapByPath =
          new HashMap<String, Collection<? extends ActionProvider>>();

    @Override @Nonnull
    public synchronized Collection<? extends Object> createCapabilities (final @Nonnull T owner)
      {
        final Object unwrapped = Wrapper.unwrap(owner);
        Class<?> clazz = unwrapped.getClass();

        if (clazz.getName().startsWith("elmobeans.proxies"))
          {
            clazz = clazz.getSuperclass();
          }

        if (clazz.getName().contains(".impl."))
          {
            clazz = clazz.getInterfaces()[0];
          }

        final String path = "/Actions/" + clazz.getName();
          
        Collection<? extends ActionProvider> result = capabilityMapByPath.get(path);

        if (result == null)
          {
            final ActionProvider actionProvider = new ActionProvider()
              {
                @Nonnull
                final Lookup lookup = Lookups.forPath(path);
                
                @Override @Nonnull
                public Action getPreferredAction()
                  {
                    return getActions().iterator().next(); // TODO
                  }

                @Override @Nonnull
                public Collection<? extends Action> getActions()
                  {
                    return lookup.lookupAll(Action.class);
                  }
              };

            result = Collections.singletonList(actionProvider);
            capabilityMapByPath.put(path, result);
          }
          
        return result;
      }
  }
