/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.forceten.application.capabilities;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProviderSupport;
import it.tidalwave.netbeans.nodes.role.ChildFactoryExceptionHandler;
import it.tidalwave.netbeans.nodes.role.RetryingChildFactoryExceptionHandler;
import it.tidalwave.geo.geocoding.GeoCoderEntity;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=CapabilitiesProvider.class)
public class NodeExceptionHandlerProviderForGeoCoderEntity extends CapabilitiesProviderSupport<GeoCoderEntity>
  {
    private final RetryingChildFactoryExceptionHandler exceptionHandler = new RetryingChildFactoryExceptionHandler();

    private final Collection<ChildFactoryExceptionHandler> nodeExceptionHandler =
            Collections.<ChildFactoryExceptionHandler>singletonList(exceptionHandler);

    public NodeExceptionHandlerProviderForGeoCoderEntity()
      {
        exceptionHandler.setMaxRetryCount(2);
        exceptionHandler.setRetryPeriod(5000);
      }

    @Override @Nonnull
    public Collection<? extends Object> createCapabilities (final @Nonnull GeoCoderEntity geoCoderEntity)
      {
        final String geoCoderName = geoCoderEntity.getGeoCoder().getName();
        return "GeoNames".equals(geoCoderName) ? nodeExceptionHandler : Collections.<Object>emptyList();
      }
  }
