/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.forceten.application.role;

import javax.annotation.Nonnull;
import javax.annotation.PostConstruct;
import javax.inject.Inject;
import org.netbeans.platformx.eventbus.api.EventBus;
import org.netbeans.platformx.eventbus.api.EventBusListener;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.Task;
import it.tidalwave.netbeans.aspect.Aspects;
import it.tidalwave.netbeans.windows.role.PresentationLocker;
import it.tidalwave.netbeans.windows.role.TopComponentHelper;
import it.tidalwave.geo.location.GeoLocationSet;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.viewer.GeoViewManager;
import it.tidalwave.geo.viewer.ZoomLevel;
import it.tidalwave.geo.viewer.FeatureManager;

/***********************************************************************************************************************
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
//@ServiceProvider(service=Object.class, path="Roles/it.tidalwave.geo.viewer.GeoViewerPresentation/XXX.instance")
public class EventBusGeoLocationListener
  {
    private static final String CLASS = EventBusGeoLocationListener.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Inject
    private TopComponentHelper topComponentHelper;

    @Inject
    private PresentationLocker presentationLocker;

    @Inject
    private GeoViewManager geoViewManager;

    @Inject
    private FeatureManager featureManager;

    @Inject
    private EventBus eventBus;

    private final EventBusListener<GeoLocationSet> geoLocationListener = new EventBusListener<GeoLocationSet>()
      {
        @Override
        public void notify (final @Nonnull GeoLocationSet geoLocationSet)
          {
            logger.fine("notify(%s)", geoLocationSet);
            
            if (geoLocationSet != null)
              {
                topComponentHelper.openAndRequestActive(); // FIXME: move to a separate role
                presentationLocker.lock("Loading data...");

                try
                  {
                    final GeoSchema geoSchema = geoLocationSet.getGeoSchema();
                    // FIXME: split the role that adds features from the one that fits the view
                    // FIXME: find a better way to apply the Aspect
            //      final Lookup transactionalAspects = Lookups.fixed(new SesameTransactionalAspectFactory()); FIXME
                    final Lookup transactionalAspects = Lookups.fixed();

                    Aspects.run(transactionalAspects, new Task<Void, RuntimeException>()
                      {
                        @Override
                        public Void run()
                          {
                            for (final GeoLocation geoLocation : geoLocationSet)
                              {
                                try
                                  {
                                    featureManager.addFeature(geoLocation, geoLocation.findCoordinate(geoSchema));
                                  }
                                catch (NotFoundException e)
                                  {
                                    // FIXME: log
                                  }
                              }

                            return null;
                          }
                      });
                    
                    geoViewManager.fitView(geoLocationSet.getSector(), ZoomLevel.REGION);
                  }
                finally
                  {
                    presentationLocker.unlock();
                  }
              }
          }
      };

    @PostConstruct
    @SuppressWarnings("PMD.UnusedPrivateMethod") @edu.umd.cs.findbugs.annotations.SuppressWarnings("UPM_UNCALLED_PRIVATE_METHOD")
    private void initialize()
      {
        eventBus.subscribe(GeoLocationSet.class, geoLocationListener);
      }
  }
