/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.forceten.application;

import org.junit.Before;
import org.junit.Test;
import javax.swing.JComponent;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.explorer.ExplorerPanel.TransitionController;
import it.tidalwave.netbeans.explorer.FadingPanelTransitionController;
import it.tidalwave.netbeans.explorer.view.EnhancedBeanTreeView;
import it.tidalwave.netbeans.nodes.role.NodeDecorator;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import it.tidalwave.netbeans.windows.role.TitleActivator;
import it.tidalwave.geo.explorer.impl.GeoExplorerTopComponent;
import it.tidalwave.geo.geocoding.node.GeoNodeWithEmblemDecorator;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;
import org.hamcrest.core.IsInstanceOf;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoExplorerTopComponentTest
  {
    private GeoExplorerTopComponent fixture;
    private EnhancedWindowManager windowManager;

    @Before
    public void setUp()
      {
        windowManager = mock(EnhancedWindowManager.class);
        MockLookup.setInstances(windowManager);
        fixture = GeoExplorerTopComponent.findInstance();
        fixture.installRoles();
      }

    // FIXME: can't be tested here - redo as a pluggable AcceptanceTest.
//    @Test
//    public void shouldNotHaveAutoSelectionStrategy()
//      {
//        final ExplorerSelectionStrategy explorerSelectionStrategy = fixture.getLookup().lookup(ExplorerSelectionStrategy.class);
//        assertThat(explorerSelectionStrategy, is(nullValue()));
//      }

    @Test
    public void shouldHaveTitleActivator()
      {
        final TitleActivator titleActivator = fixture.getLookup().lookup(TitleActivator.class);
        assertThat(titleActivator, is(not(nullValue())));
        assertThat(titleActivator, IsInstanceOf.instanceOf(TitleActivator.class));
      }

    @Test
    public void shouldHaveEmblemDecorator()
      {
        final NodeDecorator nodeDecorator = fixture.getLookup().lookup(GeoNodeWithEmblemDecorator.class);
        assertThat(nodeDecorator, is(not(nullValue())));
        assertThat(nodeDecorator, IsInstanceOf.instanceOf(GeoNodeWithEmblemDecorator.class));
      }

    @Test
    public void shouldHaveFadingPanelTransitionController()
      {
        final TransitionController transitionController = fixture.getLookup().lookup(FadingPanelTransitionController.class);
        assertThat(transitionController, is(not(nullValue())));
        assertThat(transitionController, IsInstanceOf.instanceOf(FadingPanelTransitionController.class));
      }

    @Test
    public void shouldHaveView()
      {
        final JComponent view = fixture.getLookup().lookup(EnhancedBeanTreeView.class);
        assertThat(view, is(not(nullValue())));
      }
  }