/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.examples.example2;

import javax.annotation.Nonnull;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderEntity.FactSheet;
import it.tidalwave.geo.geocoding.GeoCoderEntity.Type;
import static it.tidalwave.role.Displayable.Displayable;
import static it.tidalwave.geo.Units.*;
import static it.tidalwave.geo.GeoContainer.GeoContainer;
import static it.tidalwave.geo.geocoding.GeoCoderManager.GeoCoderManager;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.GeoCoderEntity;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.FactSheet.*;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.FactSheet.FactSheet;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class Main
  {
    public static void main (final @Nonnull String ... args)
      throws NotFoundException
      {
        final GeoCoder geoCoder = Locator.find(GeoCoderManager).findDefaultGeoCoder();
        final Range range = Range.on(new Coordinate(43.5, 6)).withRadius(km(10));

        for (final GeoCoderEntity place : range.as(GeoContainer).find(GeoCoderEntity).with(geoCoder).results())
          {
            final String displayName = place.as(Displayable).getDisplayName();
            final Type type = place.getType();
            final Coordinate coordinate = place.getCoordinate();
            final FactSheet factSheet = place.as(FactSheet);
            
            System.err.printf("%-30s %-5s: %s\n", displayName, type, coordinate);

            if (factSheet.contains(POPULATION))
              {
                System.err.printf("%42s %d\n", "population:", factSheet.get(POPULATION));
              }

            if (factSheet.contains(ELEVATION))
              {
                System.err.printf("%42s %d\n", "elevation: ", factSheet.get(ELEVATION));
              }
          }
      }
  }
