/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * A {@code GeoTag} is basically a {@link Coordinate} applied to a tagged object, plus precision information. This
 * entity is mutable and it has been designed to eventually be persistent.
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface GeoTag extends Lookup.Provider
  {
    /*******************************************************************************************************************
     *
     * Sets the {@code Coordinate} of this {@code GeoTag}.
     *
     * @param  coordinate   the coordinate
     *
     ******************************************************************************************************************/
    public void setCoordinate (@Nonnull Coordinate coordinate);

    /*******************************************************************************************************************
     *
     * Sets the {@code Coordinate} of this {@code GeoTag} and mark it as 'coarse'. A coarse {@code GeoTag} is usually
     * used for representing a reference position in a large geographic entity, such as a  province, a region or a
     * country, with no specific precision.
     *
     * @see #isCoarse()
     * @param  coordinate   the coordinate
     *
     ******************************************************************************************************************/
    public void setCoarseCoordinate (@Nonnull Coordinate coordinate);

    /*******************************************************************************************************************
     *
     * Return the {@link Coordinate} of this {@code GeoTag}.
     *
     * @return the {@code Coordinate}
     *
     ******************************************************************************************************************/
    @Nonnull
    public Coordinate getCoordinate();

    /*******************************************************************************************************************
     *
     * Returns whether this {@code GeoTag} is 'coarse', that is whether there's no precision information available.
     *
     * @see #setCoarsePosition(double, double)
     *
     * @return  {@code true} if this {@code GeoTag} is coarse
     *
     ******************************************************************************************************************/
    public boolean isCoarse();

    /*******************************************************************************************************************
     *
     * Sets the precision of this {@code GeoTag}.
     *
     * @param  precision  the precision in meter
     *
     ******************************************************************************************************************/
    public void setPrecision (@Nonnegative double precision)
      throws GeoException;

    /*******************************************************************************************************************
     *
     * Return the precision of this {@code GeoTag}.
     *
     * @return the precision in meters
     *
     ******************************************************************************************************************/
    @Nonnegative
    public double getPrecision();

    /*******************************************************************************************************************
     *
     * Returns the associated object.
     *
     * @return   the associated object
     *
     ******************************************************************************************************************/
    @CheckForNull
    public Object getBoundObject();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isValid();
  }
