/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.util.As;
import it.tidalwave.util.As.NotFoundBehaviour;
import it.tidalwave.netbeans.capabilitiesprovider.LookupFactory;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class AsSupport implements As, Lookup.Provider
  {
    @CheckForNull
    private Lookup lookup;

    @Override @Nonnull
    public <T> T as (final @Nonnull Class<T> clazz)
      {
        return as(clazz, As.Defaults.throwAsException(clazz));
      }

    @Override @Nonnull
    public <T> T as (final @Nonnull Class<T> clazz,
                     final @Nonnull NotFoundBehaviour<T> notFoundBehaviour)
      {
        final T object = getLookup().lookup(clazz);
        return (object != null) ? object : notFoundBehaviour.run(null);
      }

    @Override @Nonnull
    public synchronized Lookup getLookup()
      {
        if (lookup == null)
          {
            // FIXME: use a LazyLookup for GeoContainerImpl?
            lookup = LookupFactory.createLookup(this, Lookups.fixed(new GeoContainerImpl(this)));
          }

        return lookup;
      }
  }
