/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.impl;

import javax.annotation.Nonnull;
import it.tidalwave.geo.Coordinate;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class WGS84
  {
    public static final double A = 6378137;
    public static final double B = 6356752.3142;

    /***************************************************************************
     *
     *
     **************************************************************************/
    private WGS84()
      {
      }

    public static double distance (final @Nonnull Coordinate c1, final @Nonnull Coordinate c2)
      {
        final double h = Math.min(c1.getAltitude(), c2.getAltitude());
        return distance(c1.getLatitude(), c1.getLongitude(),
                        c2.getLatitude(), c2.getLongitude(),
                        h);
      }

    /***************************************************************************
     *
     * Parameters are in degrees.
     *
     * See http://members.shaw.ca/reastick/Geocaching/MyCaches/CirclesOrTriangles/Help/Circles%20or%20Triangles%20Cache%20-%20Calculation%20Guidance.htm
     *
     **************************************************************************/
    public static double distance (double lat1, double lon1, double lat2, double lon2, double h)
      {
        lat1 = Math.toRadians(lat1);
        lon1 = Math.toRadians(lon1);
        lat2 = Math.toRadians(lat2);
        lon2 = Math.toRadians(lon2);
        double sq1 = (lat1 - lat2) * fLat(lat1, h);
        double sq2 = (lon1 - lon2) * fLon(lat1, h);
        return Math.sqrt(sq1 * sq1 + sq2 * sq2);
      }

    /***************************************************************************
     *
     * Parameters are in radians.
     *
     **************************************************************************/
    private static double fLat (double p, double h)
      {
        double cosP = Math.cos(p);
        double sinP = Math.sin(p);
        // Looks like the AuxMath.pow() doesn't work
//        return h + (A * A * B * B) / AuxMath.pow(A * A * cosP * cosP + B * B * sinP * sinP, 1.5);
        return h + (A * A * B * B) / pow(A * A * cosP * cosP + B * B * sinP * sinP, 1.5);
      }

    private static double pow (double x, double y)
      {
        if (y != 1.5) throw new RuntimeException();
        double s = Math.sqrt(x);
        return s * s * s;
      }

    /***************************************************************************
     *
     * Parameters are in radians.
     *
     **************************************************************************/
    private static double fLon (double p, double h)
      {
        double cosP = Math.cos(p);
        double sinP = Math.sin(p);
        return (h + A * A / Math.sqrt(A * A * cosP * cosP + B * B * sinP * sinP)) * cosP;
      }
  }
