/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo;

import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class CoordinateSetTest
  {
    @Test
    public void emptySetShouldBeEmpty()
      {
        final CoordinateSet cSet = new CoordinateSet();
        assertThat(cSet.getSize(), is(0));
        final Sector sector = cSet.getSector();
        assertThat(sector, notNullValue());
        assertThat(sector.getMinLatitude(),  is(Double.NaN));
        assertThat(sector.getMaxLatitude(),  is(Double.NaN));
        assertThat(sector.getMinLongitude(), is(Double.NaN));
        assertThat(sector.getMaxLongitude(), is(Double.NaN));
        assertThat(cSet.toString(), is("CoordinateSet[empty]"));
      }

    @Test
    public void test1()
      {
        final CoordinateSet cSet = new CoordinateSet();
        final Coordinate c1 = new Coordinate(1.0, 2.0);
        final CoordinateSet cSet1 = cSet.with(c1);
        assertThat(cSet1.getSize(), is(1));
        final Sector sector = cSet1.getSector();
        assertThat(sector, notNullValue());
        assertThat(sector.getMinLatitude(),  is(1.0));
        assertThat(sector.getMaxLatitude(),  is(1.0));
        assertThat(sector.getMinLongitude(), is(2.0));
        assertThat(sector.getMaxLongitude(), is(2.0));
        assertThat(sector.isSingleCoordinate(), is(true));
        assertThat(cSet1.toString(), is("CoordinateSet[[N 1\u00b0 00' 00.000\" E 2\u00b0 00' 00.000\", 0.0] / " +
                                        "Sector[N 1\u00b0 00' 00.000\" -> N 1\u00b0 00' 00.000\", " +
                                               "E 2\u00b0 00' 00.000\" -> E 2\u00b0 00' 00.000\"]]"));
      }

    @Test
    public void test2()
      {
        final CoordinateSet cSet = new CoordinateSet();
        final Coordinate c1 = new Coordinate(10.0, 20.0);
        final Coordinate c2 = new Coordinate(15.0, 10.0);
        final CoordinateSet cSet1 = cSet.with(c1, c2);
        assertThat(cSet1.getSize(), is(2));
        final Sector sector = cSet1.getSector();
        assertThat(sector, notNullValue());
        assertThat(sector.getMinLatitude(),  is(10.0));
        assertThat(sector.getMaxLatitude(),  is(15.0));
        assertThat(sector.getMinLongitude(), is(10.0));
        assertThat(sector.getMaxLongitude(), is(20.0));
        assertThat(sector.isSingleCoordinate(), is(false));
        assertThat(cSet1.toString(), is("CoordinateSet[[N 10\u00b0 00' 00.000\" E 20\u00b0 00' 00.000\", 0.0, " +
                                                       "N 15\u00b0 00' 00.000\" E 10\u00b0 00' 00.000\", 0.0] / " +
                                        "Sector[N 15\u00b0 00' 00.000\" -> N 10\u00b0 00' 00.000\", " +
                                               "E 10\u00b0 00' 00.000\" -> E 20\u00b0 00' 00.000\"]]"));
      }

    @Test
    public void test3()
      {
        final CoordinateSet cSet = new CoordinateSet();
        final Coordinate c1 = new Coordinate(10.0, 20.0);
        final Coordinate c2 = new Coordinate(15.0, 10.0);
        final Coordinate c3 = new Coordinate(12.0, 15.0);
        final CoordinateSet cSet1 = cSet.with(c1, c2, c3);
        assertThat(cSet1.getSize(), is(3));
        final Sector sector = cSet1.getSector();
        assertThat(sector, notNullValue());
        assertThat(sector.getMinLatitude(),  is(10.0));
        assertThat(sector.getMaxLatitude(),  is(15.0));
        assertThat(sector.getMinLongitude(), is(10.0));
        assertThat(sector.getMaxLongitude(), is(20.0));
        assertThat(sector.isSingleCoordinate(), is(false));
        assertThat(cSet1.toString(), is("CoordinateSet[[N 10\u00b0 00' 00.000\" E 20\u00b0 00' 00.000\", 0.0, " +
                                                       "N 15\u00b0 00' 00.000\" E 10\u00b0 00' 00.000\", 0.0, " +
                                                       "N 12\u00b0 00' 00.000\" E 15\u00b0 00' 00.000\", 0.0] / " +
                                        "Sector[N 15\u00b0 00' 00.000\" -> N 10\u00b0 00' 00.000\", " +
                                               "E 10\u00b0 00' 00.000\" -> E 20\u00b0 00' 00.000\"]]"));
      }
  }