/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.impl;

import javax.annotation.Nonnull;
import java.util.HashSet;
import java.util.Set;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderManager;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=GeoCoderManager.class)
public class GeoCoderManagerImpl implements GeoCoderManager
  {
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Set<GeoCoder> findAllGeoCoders()
      {
        return new HashSet<GeoCoder>(Lookup.getDefault().lookupAll(GeoCoder.class));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public GeoCoder findDefaultGeoCoder()
      throws NotFoundException
      {
        final Set<GeoCoder> geoCoders = findAllGeoCoders();

        if (geoCoders.isEmpty()) // FIXME: use NotFoundException factory method
          {
            throw new NotFoundException("No GeoCoders found - please install one");
          }

        return geoCoders.iterator().next();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public GeoCoder findGeoCoderByName (final @Nonnull String name)
      throws NotFoundException
      {
        for (final GeoCoder geoCoder : findAllGeoCoders())
          {
            if (geoCoder.getName().equals(name))
              {
                return geoCoder;
              }
          }
        
        throw new NotFoundException("Not found: " + name);
      }
  }
