/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.impl.finder;

import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.Finder;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.finder.spi.GeoFinderProvider;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderManager;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=GeoFinderProvider.class)
public class CoordinateFinder extends GeoFinderProvider<GeoCoderEntity, Coordinate>
  {
    private final static String CLASS = CoordinateFinder.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final GeoCoder geoCoder;

    public CoordinateFinder()
      throws NotFoundException
      {
        super(Coordinate.class, GeoCoderEntity.class);
        geoCoder = Locator.find(GeoCoderManager.class).findDefaultGeoCoder();
      }

    @Override @Nonnull
    protected Finder<GeoCoderEntity> find (final @Nonnull Coordinate coordinate,
                                           final @Nonnull Class<? extends GeoCoderEntity> type,
                                           final @Nonnull Lookup parameters)
      {
        logger.finest("find(%s, %s, %s)", coordinate, type, parameters.lookupAll(Object.class));
        return geoCoder.findNearestEntity(coordinate);
      }
  }
