/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding;

import it.tidalwave.geo.Coordinate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderEntitySetTest
  {
    @Test
    public void testEmptyGeoCoderEntitySet()
      {
        final GeoCoderEntitySet geoCoderEntitySet = new GeoCoderEntitySet();
        
        assertThat(geoCoderEntitySet.size(), is(0));
        assertThat(geoCoderEntitySet.getSector().isVoid(), is(true));
        assertThat(geoCoderEntitySet.iterator().hasNext(), is(false));
        final String toString = String.format("GeoEntitySet@%x[0 entities, Sector[void]]",
                                              System.identityHashCode(geoCoderEntitySet));
        assertThat(geoCoderEntitySet.toString(), is(toString));
      }

    @Test
    public void testGeoCoderEntitySetWithOneEntity()
      {
        final GeoCoderEntity geoEntity = mock(GeoCoderEntity.class);
        when(geoEntity.getCoordinate()).thenReturn(new Coordinate(10, 20, 30));

        final GeoCoderEntitySet geoEntitySet = new GeoCoderEntitySet(geoEntity);
        
        assertThat(geoEntitySet.size(), is(1));
        final List<GeoCoderEntity> list = toList(geoEntitySet.iterator());
        assertThat(list.size(), is(1));
        assertThat(list.get(0), is(sameInstance(geoEntity)));
        assertThat(geoEntitySet.getSector().getNWCorner(), is(new Coordinate(10, 20)));
        assertThat(geoEntitySet.getSector().getSECorner(), is(new Coordinate(10, 20)));
        final String toString = String.format("GeoEntitySet@%x[1 entities, " +
                "Sector[N 10\u00b0 00' 00.000\" -> N 10\u00b0 00' 00.000\", E 20\u00b0 00' 00.000\" -> E 20\u00b0 00' 00.000\"]]",
                System.identityHashCode(geoEntitySet));
        assertThat(geoEntitySet.toString(), is(toString));
      }

    @Test
    public void testGeoCoderEntitySetWithTwoEntities()
      {
        final GeoCoderEntity geoEntity1 = mock(GeoCoderEntity.class);
        final GeoCoderEntity geoEntity2 = mock(GeoCoderEntity.class);
        when(geoEntity1.getCoordinate()).thenReturn(new Coordinate(10, 20, 30));
        when(geoEntity2.getCoordinate()).thenReturn(new Coordinate(-5, -7, 40));

        final GeoCoderEntitySet geoEntitySet = new GeoCoderEntitySet(geoEntity1, geoEntity2);

        assertThat(geoEntitySet.size(), is(2));
        final List<GeoCoderEntity> list = toList(geoEntitySet.iterator());
        assertThat(list.size(), is(2));
        assertThat(list.get(0), is(sameInstance(geoEntity1)));
        assertThat(list.get(1), is(sameInstance(geoEntity2)));
        assertThat(geoEntitySet.getSector().getNWCorner(), is(new Coordinate(10, -7)));
        assertThat(geoEntitySet.getSector().getSECorner(), is(new Coordinate(-5, 20)));
        final String toString = String.format("GeoEntitySet@%x[2 entities, " +
                "Sector[N 10\u00b0 00' 00.000\" -> S 5\u00b0 00' 00.000\", W 7\u00b0 00' 00.000\" -> E 20\u00b0 00' 00.000\"]]",
                System.identityHashCode(geoEntitySet));
        assertThat(geoEntitySet.toString(), is(toString));
      }

    private static <T> List<T> toList (final Iterator<T> i)
      {
        assertThat(i, is(notNullValue()));
        final List<T> list = new ArrayList<T>();

        while (i.hasNext())
          {
            list.add(i.next());
          }

        return list;
      }
  }