/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding;

import java.io.Serializable;
import org.openide.util.Lookup;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.geocoding.GeoCoderEntity.Type;
import it.tidalwave.geo.geocoding.impl.DefaultGeoCoderEntity;
import it.tidalwave.role.Displayable;
import it.tidalwave.util.Finder;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderSupportTest
  {
    public static class Capability implements Serializable
      {
      }

    private final static String GEO_CODER_ID = "http://geocoder";
    private final static String ID = "theId";
    private final static String NAME = "theName";
    private final static String CODE = "theCode";
    private final static String TYPE_AS_STRING = "theType";
    private final static Coordinate COORDINATE = new Coordinate(1, 2, 3);
    private final static Capability CAP1 = new Capability();
    private final static Capability CAP2 = new Capability();

    private GeoCoderSupport fixture;
    private GeoCoderEntity parent;

    @Before
    public void setup()
      {
        parent = mock(GeoCoderEntity.class);
        
        fixture = new GeoCoderSupport()
          {
            @Override
            public String getName()
              {
                return GEO_CODER_ID;
              }

            @Override
            public String getId()
              {
                return GEO_CODER_ID;
              }

            @Override
            public GeoCoderEntity getRoot()
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public GeoCoderEntity findGeoEntityById(String id)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<GeoCoderEntity> findChildren(String id)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<GeoCoderEntity> findNeighbours(String id)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<GeoCoderEntity> findSiblings(String id)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<String> findNearbyPostalCodes(Coordinate coordinate, float radius)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<GeoCoderEntity> findNearestEntity(Coordinate coordinate)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Finder<GeoCoderEntity> findEntities(Coordinate coordinate, float radius)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public String findCountryCode(Coordinate coordinate)
              {
                throw new UnsupportedOperationException();
              }

            @Override
            public Type typeFromString(String typeAsString)
              {
                throw new UnsupportedOperationException();
              }
          };
      }

    @Test
    public void shouldCreateEntityAndCacheInstances()
      {
        final GeoCoderEntity geoEntity = fixture.createGeoEntity(parent, ID, NAME, COORDINATE,
                                                                 CODE, TYPE_AS_STRING, 
                                                                 new Serializable[] {CAP1, CAP2});
        assertThat(geoEntity, is(notNullValue()));
        assertThat(geoEntity, is(instanceOf(DefaultGeoCoderEntity.class)));
        assertThat(geoEntity.getId(), is(ID));
        assertThat(geoEntity.getCoordinate(), is(COORDINATE));
        assertThat(geoEntity.getTypeAsString(), is(TYPE_AS_STRING));
        assertThat(geoEntity.as(Displayable.class).getDisplayName(), is(NAME));

        final GeoCoderEntity geoEntity2 = fixture.createGeoEntity(parent, ID, NAME, COORDINATE,
                                                                  CODE, TYPE_AS_STRING,
                                                                  new Serializable[] {CAP1, CAP2});
        final GeoCoderEntity geoEntity3 = fixture.createGeoEntity(parent, ID, NAME, COORDINATE,
                                                                  CODE, TYPE_AS_STRING,
                                                                  new Serializable[] {CAP1, CAP2});
        assertThat(geoEntity2, is(sameInstance(geoEntity)));
        assertThat(geoEntity3, is(sameInstance(geoEntity)));
      }

    @Test
    public void shouldCreateLookupOnlyOnce()
      {
        final Lookup lookup = fixture.getLookup();

        for (int i = 0; i < 10; i++)
          {
            assertThat(fixture.getLookup(), is(sameInstance(lookup)));
          }
      }
  }