/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.impl;

import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderManager;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.test.MockLookup;
import java.util.Arrays;
import java.util.Collection;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderManagerTest
  {
    private GeoCoder geoCoder1;
    private GeoCoder geoCoder2;
    private GeoCoder geoCoder3;

    private GeoCoderManager fixture;
    
//    @BeforeClass
//    public static void setupMockGeoCoders()
//      {
//      }

    @Before
    public void setup()
      {
        fixture = new GeoCoderManagerImpl();
        geoCoder1 = mock(GeoCoder.class);
        geoCoder2 = mock(GeoCoder.class);
        geoCoder3 = mock(GeoCoder.class);
        when(geoCoder1.getName()).thenReturn("GeoCoder1");
        when(geoCoder2.getName()).thenReturn("GeoCoder2");
        when(geoCoder3.getName()).thenReturn("GeoCoder3");
        MockLookup.setInstances(geoCoder1, geoCoder2, geoCoder3);
      }

    @Test
    public void testLocator()
      {
        final GeoCoderManager geoCoderManager = Locator.find(GeoCoderManager.class);
        assertThat(geoCoderManager, is(notNullValue()));
        assertThat(geoCoderManager, instanceOf(GeoCoderManagerImpl.class));
      }

    @Test
    public void shouldFindAllGeoCoders()
      {
        final Collection<GeoCoder> geoCoders = fixture.findAllGeoCoders();
        assertThat(geoCoders.size(), is(3));
        assertThat(geoCoders.contains(geoCoder1), is(true));
        assertThat(geoCoders.contains(geoCoder2), is(true));
        assertThat(geoCoders.contains(geoCoder3), is(true));
      }

    @Test
    public void shouldFindTheDefaultGeoCoder()
      throws NotFoundException
      {
        final GeoCoder defaultGeoCoder = fixture.findDefaultGeoCoder();
        assertThat(defaultGeoCoder, is(notNullValue()));
        assertThat(Arrays.asList(geoCoder1, geoCoder2, geoCoder3).contains(defaultGeoCoder), is(true));
      }

    @Test
    public void shouldFindGeoCoderByName()
      throws NotFoundException
      {
        final GeoCoder foundGeoCoder1 = fixture.findGeoCoderByName("GeoCoder1");
        final GeoCoder foundGeoCoder2 = fixture.findGeoCoderByName("GeoCoder2");
        final GeoCoder foundGeoCoder3 = fixture.findGeoCoderByName("GeoCoder3");
        assertThat(foundGeoCoder1, is(sameInstance(geoCoder1)));
        assertThat(foundGeoCoder2, is(sameInstance(geoCoder2)));
        assertThat(foundGeoCoder3, is(sameInstance(geoCoder3)));
      }

    @Test(expected=NotFoundException.class)
    public void shouldThrowExceptionForNotFoundGeoCoder()
      throws NotFoundException
      {
        fixture.findGeoCoderByName("BADGEOCODER");
      }

    @Test(expected=NotFoundException.class)
    public void shouldThrowNotFoundExceptionWithNoGeoCoders()
      throws NotFoundException
      {
        MockLookup.setInstances();
        fixture.findDefaultGeoCoder();
      }
  }