/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node;

import javax.annotation.Nonnull;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import it.tidalwave.role.IconProvider;
import it.tidalwave.netbeans.nodes.role.NodeDecorator;
import it.tidalwave.netbeans.nodes.LookupFilterDecoratorNode;
import it.tidalwave.netbeans.nodes.LookupFilterDecoratorNode.LookupFilter;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderEntity.EmblemFactory;
import it.tidalwave.geo.geocoding.node.impl.EmblemIconProviderDecorator;

/***********************************************************************************************************************
 *
 * A decorator for {@link Node}s which attaches the emblem of the {@link GeoCoderEntity} as it is provided by its
 * {@link EmblemFactory}.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoNodeWithEmblemDecorator implements NodeDecorator
  {
    private final LookupFilter lookupFilter = new LookupFilter()
      {
        @Override
        public Lookup filter (final @Nonnull Lookup lookup)
          {
            final IconProvider iconProvider = lookup.lookup(IconProvider.class);
            final EmblemIconProviderDecorator newIconProvider = new EmblemIconProviderDecorator(iconProvider, lookup);
            return new ProxyLookup(Lookups.exclude(lookup, IconProvider.class), Lookups.singleton(newIconProvider));
          }
      };

    @Override @Nonnull
    public Node decorate (final @Nonnull Node node)
      {
        return new LookupFilterDecoratorNode(node, lookupFilter);
      }
  }
