/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node.impl;

import java.util.Collection;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.netbeans.nodes.role.NodeDelegateFactory;
import org.junit.Before;
import org.junit.Test;
import org.hamcrest.Matcher;
import static org.junit.Assert.*;
import static org.junit.matchers.JUnitMatchers.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.Mockito.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderEntityNodeDelegateFactoryCapabilityProviderTest extends TestSupport
  {
    private GeoCoderEntityNodeDelegateFactoryCapabilityProvider fixture;
    private GeoCoderEntity geoCoderEntity;

    @Before
    public void setupFixture()
      {
        fixture = new GeoCoderEntityNodeDelegateFactoryCapabilityProvider();
        final GeoCoderEntity rootGeoCoderEntity = createMockGeoCoderEntity("root");
        when(geoCoder.getRoot()).thenReturn(rootGeoCoderEntity);

        geoCoderEntity = createMockGeoCoderEntity("mockGeoCoderEntity");
      }

    @Test
    public void shouldBeRegisteredInTheDefaultLookup()
      {
        Collection<? extends CapabilitiesProvider> capabilities = Lookup.getDefault().lookupAll(CapabilitiesProvider.class);
        assertThat(capabilities, is(notNullValue()));
        final Matcher<Iterable<? extends CapabilitiesProvider>> hasItem = (Matcher)hasItem(instanceOf(GeoCoderEntityNodeDelegateFactoryCapabilityProvider.class));
        assertThat(capabilities, hasItem);
      }

    @Test
    public void shouldDeclareToManageGeoSchema()
      {
        assertEquals(GeoCoderEntity.class, fixture.getManagedClass());
//        assertThat(fixture.getManagedClass(), is(eq(GeoCoderEntity.class))); FIXME
      }

    @Test
    public void shouldCreateANodeDelegateFactoryWhichReturnsANodeWrappingTheRootGeoLocation()
      throws NotFoundException
      {

        final Collection<? extends Object> capabilities = fixture.createCapabilities(geoCoderEntity);
        assertThat(capabilities, is(notNullValue()));
        assertThat(capabilities.size(), is(1));
        assertThat(capabilities.iterator().next(), instanceOf(NodeDelegateFactory.class));

        final NodeDelegateFactory nodeDelegateFactory = (NodeDelegateFactory)capabilities.iterator().next();
        final Node node = nodeDelegateFactory.createNodeDelegate();
        assertThat(node, is(notNullValue()));
        final Lookup lookup = node.getLookup();
        assertThat(lookup, is(notNullValue()));
        final GeoCoderEntity lookedUpGeoCoderEntity = lookup.lookup(GeoCoderEntity.class);
        assertThat(lookedUpGeoCoderEntity, is(sameInstance(geoCoderEntity)));
      }

    @Test
    public void shouldCreateNoLookups()
      {
        final Collection<? extends Lookup> lookups = fixture.createLookups(geoCoderEntity);
        
        assertThat(lookups, is(notNullValue()));
        assertThat(lookups.isEmpty(), is(true));
      }
  }