/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node.impl;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.List;
import javax.swing.Action;
import org.openide.util.Lookup;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import it.tidalwave.util.spi.SimpleFinderSupport;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderEntityNodeTest extends TestSupport
  {
    private GeoCoderEntityNode node;
    private GeoCoderEntity parentGeoCoderEntity;
    private GeoCoderEntity childGeoCoderEntity1;
    private GeoCoderEntity childGeoCoderEntity2;
    private List<? extends Action> actions;
    private Action preferredAction;

    @Before
    public void setUp()
      {
        actions = Arrays.asList(mock(Action.class), mock(Action.class), mock(Action.class));
        parentGeoCoderEntity = createMockGeoCoderEntity("Italy");
        when(parentGeoCoderEntity.getId()).thenReturn("ID_Italy");
        preferredAction = mock(Action.class);

        node = new GeoCoderEntityNode(parentGeoCoderEntity);
        
        childGeoCoderEntity1 = createMockGeoCoderEntity("entity1");
        childGeoCoderEntity2 = createMockGeoCoderEntity("entity2");
        when(childGeoCoderEntity1.getLookup()).thenReturn(Lookup.EMPTY);
        when(childGeoCoderEntity2.getLookup()).thenReturn(Lookup.EMPTY);
        
//        when(parentGeoCoderEntity.findAllChildren()).thenReturn(Arrays.asList(c1, c2)); FIXME
        when(parentGeoCoderEntity.findChildren()).thenReturn(new SimpleFinderSupport<GeoCoderEntity>("mock")
          {
            @Override @Nonnull
            protected List<GeoCoderEntity> computeResults()
              {
                return Arrays.asList(childGeoCoderEntity1, childGeoCoderEntity2);
              }
          });
      }

    @Test
    public void mustReturnChildrenNodesThatWrapsChildrenGeoCoderEntities()
      throws InterruptedException
      {
        final Children children = node.getChildren();
        children.getNodes(); // triggers loading
        Thread.sleep(1000);  // waits for the "please wait..." node to disappear
        final Node[] nodes = children.getNodes();

        assertThat(nodes, is(notNullValue()));
        assertThat(nodes.length, is(2));
        assertThat(nodes[0].getLookup().lookup(GeoCoderEntity.class), is(sameInstance(childGeoCoderEntity1)));
        assertThat(nodes[1].getLookup().lookup(GeoCoderEntity.class), is(sameInstance(childGeoCoderEntity2)));
      }

    @Test
    public void mustHaveGeoNodeInLookup()
      {
        final Lookup lookup = node.getLookup();
        assertThat(lookup, is(not(nullValue())));
        final GeoCoderEntity lookedUpGeoEntity = lookup.lookup(GeoCoderEntity.class);
        assertThat(lookedUpGeoEntity, sameInstance(parentGeoCoderEntity));
      }

    @Test
    public void mustBeDestroyableAndRenamableButNotCopiableOrCuttable()
      {
        assertThat("canCopy",    node.canCopy(),    is(false));
        assertThat("canCut",     node.canCut(),     is(false));
        assertThat("canDestroy", node.canDestroy(), is(false));
        assertThat("canRename",  node.canRename(),  is(false));
      }

//    @Test
//    public void mustHaveTheSameNameAsTheGeoCoderEntity()
//      {
//        final String name = node.getName();
//        assertThat(name, equalTo("ID_Italy"));
//      }

    @Test
    public void mustHaveTheSameDisplayNameAsTheGeoCoderEntity()
      {
        final String displayName = node.getDisplayName();
        assertThat(displayName, equalTo("Italy"));
      }

    @Test
    public void mustHaveANullHTMLDisplayName()
      {
        final String htmlDisplayName = node.getHtmlDisplayName();
        assertThat(htmlDisplayName, is(nullValue()));
      }
  }