/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node.impl;

import java.util.Collection;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.nodes.role.RootNodeProvider;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import org.hamcrest.Matcher;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.junit.matchers.JUnitMatchers.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.Mockito.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoCoderRootNodeProviderCapabilityProviderTest extends TestSupport
  {
    private GeoCoderRootNodeProviderCapabilityProvider fixture;

    @Before
    public void setupFixture()
      {
        fixture = new GeoCoderRootNodeProviderCapabilityProvider();
      }

    @Test
    public void shouldBeRegisteredInTheDefaultLookup()
      {
        Collection<? extends CapabilitiesProvider> capabilities = Lookup.getDefault().lookupAll(CapabilitiesProvider.class);
        assertThat(capabilities, is(notNullValue()));
        final Matcher<Iterable<? extends CapabilitiesProvider>> hasItem = (Matcher)hasItem(instanceOf(GeoCoderRootNodeProviderCapabilityProvider.class));
        assertThat(capabilities, hasItem);
      }

    @Test
    public void shouldDeclareToManageGeoSchema()
      {
        assertEquals(GeoCoder.class, fixture.getManagedClass());
//        assertThat(fixture.getManagedClass(), is(eq(GeoCoder.class))); FIXME
      }

    @Test
    public void shouldCreateARootNodeProviderWhichReturnsANodeWrappingTheRootGeoLocation()
      throws NotFoundException
      {
        final GeoCoderEntity rootGeoCoderEntity = createMockGeoCoderEntity("root");
        when(geoCoder.getRoot()).thenReturn(rootGeoCoderEntity);
        
        final Collection<? extends Object> capabilities = fixture.createCapabilities(geoCoder);
        
        assertThat(capabilities, is(notNullValue()));
        assertThat(capabilities.size(), is(1));
        assertThat(capabilities.iterator().next(), instanceOf(RootNodeProvider.class));
        final RootNodeProvider rootNodeProvider = (RootNodeProvider)capabilities.iterator().next();
        final Node rootNode = rootNodeProvider.getRootNode();
        assertThat(rootNode, is(notNullValue()));
        final Lookup lookup = rootNode.getLookup();
        assertThat(lookup, is(notNullValue()));
        final GeoCoderEntity lookedUpGeoCoderEntity = lookup.lookup(GeoCoderEntity.class);
        assertThat(lookedUpGeoCoderEntity, is(sameInstance(rootGeoCoderEntity)));
      }

    @Test
    public void shouldCreateNoLookups()
      {
        final Collection<? extends Lookup> lookups = fixture.createLookups(geoCoder);
        
        assertThat(lookups, is(notNullValue()));
        assertThat(lookups.isEmpty(), is(true));
      }

    @Test
    public void shouldCreateASingleNodeFactoryForEachSchema()
      throws NotFoundException
      {
        final GeoCoder geoCoder2 = mock(GeoCoder.class);
        final GeoCoderEntity rootGeoCoderEntity = createMockGeoCoderEntity("root");
        when(geoCoder2.getRoot()).thenReturn(rootGeoCoderEntity);

//        assertThat(fixture.nodeFactoryMapByGeoCoder.size(), is(0));
//
//        for (int i = 0; i < 10; i++)
//          {
//            ((RootNodeProvider)fixture.createCapabilities(geoCoder).iterator().next()).getRootNode();
//          }
//
//        assertThat(fixture.nodeFactoryMapByGeoCoder.size(), is(1));
//
//        for (int i = 0; i < 10; i++)
//          {
//            ((RootNodeProvider)fixture.createCapabilities(geoCoder2).iterator().next()).getRootNode();
//          }
//
//        assertThat(fixture.nodeFactoryMapByGeoCoder.size(), is(2));
      }
  }