/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.node.impl;

import javax.annotation.Nonnull;
import it.tidalwave.util.AsException;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.Displayable;
import it.tidalwave.role.LocalizedDisplayable;
import it.tidalwave.role.spi.DefaultDisplayable;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderManager;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.openide.util.lookup.Lookups;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.Mockito.*;
import static org.mockito.Mockito.any;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TestSupport
  {
    protected static GeoCoder geoCoder;

    @BeforeClass
    public static void installMockGeoNodeManager()
      throws NotFoundException
      {
        geoCoder = mock(GeoCoder.class);
        MockLookup.setInstances(geoCoder);
        assertThat(Locator.find(GeoCoderManager.class).findDefaultGeoCoder(), sameInstance(geoCoder));
      }

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(TestSupport.class);
      }

    @AfterClass
    public static void uninstallMockGeoNodeManager()
      {
        MockLookup.setInstances();
      }

    @Nonnull
    public static GeoCoderEntity createMockGeoCoderEntity (final @Nonnull String name)
      {
        final LocalizedDisplayable displayable = new DefaultDisplayable(name, "mock");
        final GeoCoderEntity geoCoderEntity = mock(GeoCoderEntity.class);
        when(geoCoderEntity.getGeoCoder()).thenReturn(geoCoder);
        when(geoCoderEntity.getLookup()).thenReturn(Lookups.fixed(displayable));
        
        when(geoCoderEntity.as(any(Class.class))).thenAnswer(new Answer<Object>() 
          {
            @Override @Nonnull
            public Object answer (final @Nonnull InvocationOnMock invocation) 
              {
                final Class<?> roleClass = (Class<?>)invocation.getArguments()[0];
                
                if (roleClass.equals(Displayable.class) || roleClass.equals(LocalizedDisplayable.class))
                  {
                    return displayable;
                  }
        
                throw new AsException((Class<?>)roleClass);
              }
          });
        
        return geoCoderEntity;
      }
  }
