/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.explorer;

import javax.annotation.concurrent.ThreadSafe;
import org.openide.util.Lookup;
import it.tidalwave.netbeans.util.Locator.ImplementedBy;
import it.tidalwave.geo.explorer.impl.GeoExplorerTopComponent;

/***********************************************************************************************************************
 *
 * <p>This component provides a visual component for navigating inside a hierarchy of geographical entities, rendered as 
 * a tree.
 * 
 * <!-- img src="GeoExplorerPresentation.png"/ -->
 *
 * This component does not expose a single interface for services; instead, it works as a factory where specific
 * services can be looked up.
 *
 * @has - - - org.openide.util.Lookup
 * @has - - - it.tidalwave.netbeans.role.DataLoader
 * @has - - - it.tidalwave.netbeans.explorer.role.ExplorerSelectionStrategy
 * @has - - - it.tidalwave.geo.explorer.role.SelectionHandler
 * @has - - - it.tidalwave.netbeans.windows.role.TopComponentDataLoaderStrategy
 * @has - - - it.tidalwave.netbeans.windows.role.NodeActivator
 * @has - - - it.tidalwave.netbeans.nodes.NodeDecorator
 * @has - - - it.tidalwave.netbeans.explorer.ExplorerPanel
 * @has - - - org.openide.explorer.ExplorerManager
 * @has - - - it.tidalwave.netbeans.explorer.SingleExplorerGUIBuilder
 *
 * @match class it.tidalwave.netbeans.nodes.NodeDecorator
 * |it.tidalwave.netbeans.windows.role.NodeDataLoader|it.tidalwave.netbeans.windows.role.NodeActivator
 * @opt operations
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ThreadSafe @ImplementedBy(GeoExplorerTopComponent.class)
public interface GeoExplorerPresentation extends Lookup.Provider
  {
    public static final String PREFERRED_ID = "it.tidalwave.geo.explorer.GeoExplorerPresentation";
    public static final String PATH = "Roles/" + PREFERRED_ID + "/";

    public static final String ROLE_DATA_LOADER = PATH + "DataLoader.instance";
    public static final String ROLE_DATA_LOADER_STRATEGY = PATH + "DataLoaderStrategy.instance";
    public static final String ROLE_EXPLORER_MANAGER = PATH + "ExplorerManager.instance";
    public static final String ROLE_EXPLORER_PANEL = PATH + "ExplorerPanel.instance";
    public static final String ROLE_GUI_BUILDER = PATH + "GUIBuilder.instance";
    public static final String ROLE_NODE_ACTIVATOR = PATH + "NodeActivator.instance";
    public static final String ROLE_NODE_VIEW = PATH + "NodeView.instance";
    public static final String ROLE_SELECTION_HANDLER = PATH + "SelectionHandler.instance";
    public static final String ROLE_SELECTION_STRATEGY = PATH + "SelectionStrategy.instance";
  }
