/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.explorer.impl;

import javax.annotation.Nonnull;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.geo.explorer.GeoExplorerPresentation;
import it.tidalwave.netbeans.windows.EnhancedTopComponent;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public final class GeoExplorerTopComponent extends EnhancedTopComponent implements GeoExplorerPresentation
  {
    private static final String CLASS = GeoExplorerTopComponent.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private static GeoExplorerTopComponent instance;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected GeoExplorerTopComponent()
      {
        setName(NbBundle.getMessage(GeoExplorerTopComponent.class, "CTL_GeoExplorerTopComponent"));
//        PropertyManager.configure(this, "GeoExplorerTopComponent");
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public String preferredID()
      {
        return PREFERRED_ID;
      }
    
    /*******************************************************************************************************************
     *
     * Gets default instance. Do not use directly: reserved for *.settings files only,  i.e. deserialization routines;
     * otherwise you could get a non-deserialized instance. To obtain the singleton instance, use {@link findInstance}.
     *
     ******************************************************************************************************************/
    @Nonnull
    public static synchronized GeoExplorerTopComponent getDefault() 
      {
        if (instance == null) 
          {
            instance = new GeoExplorerTopComponent();
          }
        
        return instance;
      }

    /*******************************************************************************************************************
     *
     * Obtain the GeoExplorerTopComponent instance. Never call {@link #getDefault} directly!
     *
     ******************************************************************************************************************/
    @Nonnull
    public static synchronized GeoExplorerTopComponent findInstance() 
      {
        final TopComponent component = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        
        if (component == null)
          {
            logger.warning("Cannot find %s component. It will not be located properly in the window system.",
                           PREFERRED_ID);
            return getDefault();
          }
        
        if (component instanceof GeoExplorerTopComponent)
          {
            return (GeoExplorerTopComponent)component;
          }
        
        logger.warning("There seem to be multiple components with the '%s' ID. That is a potential source of errors and " +
                       "unexpected behavior.", PREFERRED_ID);
        return getDefault();
      }
  }
