/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.explorer.role;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.util.Collection;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderEntitySet;

/***********************************************************************************************************************
 *
 * Specializations of this class handle the selection of one or more {@link GeoEntity} instances and act in
 * consequence, by implementing the method 
 * {@link #handleSelection(it.tidalwave.geo.geocoding.GeoEntitySet) handleSelection()}.
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ThreadSafe
public abstract class SelectionHandler
  {
    /*******************************************************************************************************************
     *
     * This method must perform the concrete action to react to a selection.
     *
     * @param   geoEntitySet   the {@link GeoEntitySet} representing the selection.
     *
     ******************************************************************************************************************/
    public abstract void handleSelection (@Nonnull GeoCoderEntitySet geoEntitySet);

    /*******************************************************************************************************************
     *
     * This method calls {@link #handleSelection(it.tidalwave.geo.geocoding.GeoEntitySet)}.
     *
     * @param   geoEntities    the selected {@link GeoEntity} instances
     *
     ******************************************************************************************************************/
    public final void handleSelection (@Nonnull Collection<GeoCoderEntity> geoEntities)
      {
        handleSelection(new GeoCoderEntitySet(geoEntities));
      }
  }
