/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.explorer.impl;

import java.awt.Component;
import javax.swing.JComponent;
import org.openide.explorer.ExplorerManager;
import org.openide.windows.TopComponent;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.netbeans.role.DataLoader;
import it.tidalwave.netbeans.windows.role.NodeActivator;
import it.tidalwave.netbeans.windows.role.LazyDataLoaderStrategy;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import it.tidalwave.netbeans.explorer.ExplorerPanel;
import it.tidalwave.netbeans.explorer.role.ExplorerSelectionStrategy;
import it.tidalwave.netbeans.explorer.view.EnhancedBeanTreeView;
import it.tidalwave.geo.explorer.role.SelectionHandler;
import it.tidalwave.geo.explorer.impl.role.AutoSelectionStrategy;
import it.tidalwave.geo.explorer.impl.role.EventBusSelectionHandler;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.hamcrest.core.IsInstanceOf;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import it.tidalwave.netbeans.util.test.MockLookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoExplorerTopComponentTest
  {
    private static final String ID = "it.tidalwave.geo.explorer.GeoExplorerPresentation";
    private GeoExplorerTopComponent fixture;
    private EnhancedWindowManager windowManager;

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(TestLoggerSetup.class);
      }

    @Before
    public void setup()
      {
        windowManager = mock(EnhancedWindowManager.class);
        MockLookup.setInstances(windowManager);
        fixture = GeoExplorerTopComponent.findInstance();
        fixture.installRoles();
      }

//    @Test
//    public void testInitialize()
//      {
//        System.out.println("initialize");
//        String message = "";
//        Initializer initializer = null;
//        fixture.initialize(message, initializer);
//        // TODO review the generated test code and remove the default call to fail.
//        fail("The test case is a prototype.");
//      }

    @Test
    public void shouldHaveSelectionHandler()
      {
        final SelectionHandler selectionHandler = fixture.getLookup().lookup(SelectionHandler.class);
        assertThat(selectionHandler, is(not(nullValue())));
        assertThat(selectionHandler, IsInstanceOf.instanceOf(EventBusSelectionHandler.class));
      }

    @Test
    public void shouldHaveAutoSelectionStrategy()
      {
        final ExplorerSelectionStrategy selectionStrategy = fixture.getLookup().lookup(AutoSelectionStrategy.class);
        assertThat(selectionStrategy, is(not(nullValue())));
        assertThat(selectionStrategy, IsInstanceOf.instanceOf(AutoSelectionStrategy.class));
      }

    @Test
    public void shouldHaveDataLoader()
      {
        final DataLoader dataLoader = fixture.getLookup().lookup(DataLoader.class);
        assertThat(dataLoader, is(not(nullValue())));
        assertThat(dataLoader, IsInstanceOf.instanceOf(DataLoader.class));
      }

    @Test
    public void shouldHaveLazyDataLoadRoleStrategy()
      {
        final LazyDataLoaderStrategy lazyDataLoaderStrategy = fixture.getLookup().lookup(LazyDataLoaderStrategy.class);
        assertThat(lazyDataLoaderStrategy, is(not(nullValue())));
        assertThat(lazyDataLoaderStrategy, IsInstanceOf.instanceOf(LazyDataLoaderStrategy.class));
      }

    @Test
    public void shouldHaveNodeActivator()
      {
        final NodeActivator nodeActivator = fixture.getLookup().lookup(NodeActivator.class);
        assertThat(nodeActivator, is(not(nullValue())));
        assertThat(nodeActivator, IsInstanceOf.instanceOf(NodeActivator.class));
      }

    @Test
    public void shouldHaveTopComponent()
      {
        final TopComponent component = fixture.getLookup().lookup(TopComponent.class);
        assertThat(component, is(sameInstance((Component)fixture)));
      }

    @Test
    public void shouldHaveExplorerManager()
      {
        final ExplorerManager explorerManager = fixture.getLookup().lookup(ExplorerManager.class);
        assertThat(explorerManager, is(not(nullValue())));
      }

    @Test
    public void shouldHaveExplorerPanel()
      {
        final ExplorerPanel explorerPanel = fixture.getLookup().lookup(ExplorerPanel.class);
        assertThat(explorerPanel, is(not(nullValue())));
      }

    @Test
    public void shouldHaveView()
      {
        final JComponent view1 = fixture.getLookup().lookup(EnhancedBeanTreeView.class);
        assertThat(view1, is(not(nullValue())));
        final String path = "/Roles/" + ID;
        final Lookup forPath = Lookups.forPath(path);
        final JComponent view2 = forPath.lookup(EnhancedBeanTreeView.class);
        assertThat(view2, is(not(nullValue())));
      }

    @Test
    public void testGetPersistenceType()
      {
        assertThat(fixture.getPersistenceType(), is(TopComponent.PERSISTENCE_ALWAYS));
      }

    @Test
    public void testPreferredID()
      {
        assertThat(fixture.preferredID(), is(ID));
      }
//
//    @Test
//    public void testComponentShowing()
//      {
//        fixture.componentShowing();
//        // TODO review the generated test code and remove the default call to fail.
//        fail("The test case is a prototype.");
//      }
//
//    @Test
//    public void testComponentActivated()
//      {
//        fixture.componentActivated();
//        verify(windowManager).setTitle(argThat(is("Geo")));
//        verifyNoMoreInteractions(windowManager);
//      }
//
//    @Test
//    public void testComponentDeactivated()
//      {
//        fixture.componentActivated();
//        verify(windowManager).setTitle(argThat(is("Geo")));
//        fixture.componentDeactivated();
//        verify(windowManager).setTitle(argThat(is(nullValue(String.class))));
//        verifyNoMoreInteractions(windowManager);
//      }
  }