/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.finder.spi;

import javax.annotation.Nonnull;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.Finder;
import it.tidalwave.util.NotFoundException;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class GeoFinderProvider<T, O>
  {
    private final static String CLASS = GeoFinderProvider.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);
    
    @Nonnull
    private final Class<O> ownerClass;

    @Nonnull
    private final Class<T> targetClass;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static <T, O> Finder<T> findFinder (final @Nonnull O owner, 
                                               final @Nonnull Class<T> type,
                                               final @Nonnull Lookup parameters)
      throws NotFoundException
      {
        logger.fine("findFinder(%s, %s, %s)", owner, type, parameters.lookupAll(Object.class));
        
        for (final GeoFinderProvider<?, Object> finderProvider : Lookup.getDefault().lookupAll(GeoFinderProvider.class))
          {
            if (finderProvider.ownerClass.isAssignableFrom(owner.getClass()) &&
                finderProvider.targetClass.isAssignableFrom(type))
              {
                return ((GeoFinderProvider<T, O>)finderProvider).find(owner, type, parameters);
              }
          }

        throw new NotFoundException(String.format("No finder for %s, type: %s", owner, type));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected GeoFinderProvider (final @Nonnull Class<O> ownerClass, final @Nonnull Class<T> targetClass)
      {
        this.ownerClass = ownerClass;
        this.targetClass = targetClass;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected abstract Finder<T> find (final @Nonnull O owner, 
                                       final @Nonnull Class<? extends T> type,
                                       final @Nonnull Lookup lookup);
  }
