/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location;

import javax.annotation.Nonnull;
import java.util.SortedSet;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface GeoSchemaManager
  {
    public final static Class<GeoSchemaManager> GeoSchemaManager = GeoSchemaManager.class;

    /*******************************************************************************************************************
     *
     * Returns a {@link GeoSchema} given its name.
     *
     * @param  name  the name
     * @return       the {@code GeoSchema}
     * @throws       {@link NotFoundException} if no schema with that name exists.
     *
     ******************************************************************************************************************/
    @Nonnull
    public GeoSchema findSchemaByName (@Nonnull String name)
      throws NotFoundException;

    /*******************************************************************************************************************
     *
     * Returns the set of names of available {@link GeoSchema}s.
     *
     * @return       the {@code GeoSchema} names
     *
     ******************************************************************************************************************/
    @Nonnull
    public SortedSet<String> getSchemaNames();
  }
