/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import javax.annotation.Nonnull;
import javax.xml.namespace.QName;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.As;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.semantic.EntityFactory;
import static it.tidalwave.semantic.SkosConcept.SkosConcept;
import static it.tidalwave.semantic.OwlThing.OwlThing;
import static it.tidalwave.geo.location.elmo.impl.GeoCoderEntityProxy.GeoCoderEntityProxy;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ElmoGeoCoderEntityProxyFactory 
  {
    private final static String CLASS = ElmoGeoCoderEntityProxyFactory.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    private static final EntityFactory<GeoCoderEntityProxy, ElmoGeoCoderEntityProxy> FACTORY =
                         EntityFactory.<GeoCoderEntityProxy, ElmoGeoCoderEntityProxy>
                                      forInterface(GeoCoderEntityProxy.class).
                                      withImplementation(ElmoGeoCoderEntityProxy.class).
                                      andRoles(OwlThing, SkosConcept).create();

    // FIXME: make it weak. Not identity, since QName is not flyweight
//    private static final HashMap<QName, GeoCoderEntityProxy> FLYWEIGHT_CACHE =
//            new HashMap<QName, GeoCoderEntityProxy>();

    /*******************************************************************************************************************
     *
     * Finds or creates a new GeoNode for the given GeoEntity.
     *
     ******************************************************************************************************************/
    @Nonnull
    public synchronized static GeoCoderEntityProxy findOrCreate (final @Nonnull GeoSchema geoSchema,
                                                                 final @Nonnull GeoCoderEntity gcEntity)
      {
        logger.fine("findOrCreate(%s, %s)", geoSchema, gcEntity);

        final QName qName = gcEntity.getLookup().lookup(QName.class);

        if (qName == null)
          {
            final String message = String.format("geoEntity does not contain a QName: %s - Lookup contents: %s",
                                                 gcEntity, gcEntity.getLookup().lookupAll(Object.class));
            throw new IllegalArgumentException(message);
          }

        logger.finer(">>>> qName: %s", qName);
//        GeoCoderEntityProxy gcEntityProxy = null;
////        GeoCoderEntityProxy gcEntityProxy = (ElmoGeoCoderEntityProxy)FACTORY.find(qName);
////        GeoCoderEntityProxy gcEntityProxy = FACTORY.find(qName);
//        Object object = FACTORY.find(qName);
//
//        if (object instanceof ElmoGeoCoderEntityProxy)
//          {
//            gcEntityProxy = (GeoCoderEntityProxy)object;
//          }
//        else // FIXME: why it is not null? Why find() always create an Entity if not found?
////        if (gcEntityProxy == null)
//          {
//            logger.warning("FOUND %s / %s", object.getClass(), Arrays.toString(object.getClass().getInterfaces()));
//            logger.finer(">>>> not found, creating it");
//            gcEntityProxy = FACTORY.create(qName, new EntityFactory.EntityInitializer<ElmoGeoCoderEntityProxy>()
//              {
//                @Override
//                public void initialize (final @Nonnull ElmoGeoCoderEntityProxy entity)
//                  {
//                    entity.init(gcEntity);
//                  }
//              });
//          }

        final GeoCoderEntityProxy gcEntityProxy = FACTORY.findOrCreate(qName, new EntityFactory.EntityInitializer<ElmoGeoCoderEntityProxy>()
          {
            @Override
            public void initialize (final @Nonnull ElmoGeoCoderEntityProxy entity)
              {
                entity.init(gcEntity);
              }
          });
          
        gcEntityProxy.setBindings(geoSchema);
        logger.finer(">>>> returning: %s", gcEntityProxy);

        return gcEntityProxy;

//        return flyWeight(gcEntityProxy);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static synchronized GeoCoderEntityProxy findSame (final @Nonnull As object, final @Nonnull GeoSchema geoSchema)
      throws NotFoundException
      {
        final GeoCoderEntityProxy gcEntityProxy = object.as(OwlThing).findSameAs().ofType(GeoCoderEntityProxy).result(); 
        final GeoCoderEntityProxy fwgcEntityProxy = gcEntityProxy;
//        final GeoCoderEntityProxy fwgcEntityProxy = flyWeight(gcEntityProxy);

//        if (fwgcEntityProxy == gcEntityProxy) // a new entry
          {
            fwgcEntityProxy.setBindings(geoSchema);
          }

        return fwgcEntityProxy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
//    @Nonnull
//    public static synchronized GeoCoderEntityProxy flyWeight (final @Nonnull GeoCoderEntityProxy gcEntityProxy)
//      {
//        final QName qName = gcEntityProxy.as(Entity).getQName();
//        GeoCoderEntityProxy fwgcrEntityProxy = FLYWEIGHT_CACHE.get(qName);
//
//        if (fwgcrEntityProxy == null)
//          {
//            fwgcrEntityProxy = gcEntityProxy;
//            FLYWEIGHT_CACHE.put(qName, fwgcrEntityProxy);
//          }
//
//        return fwgcrEntityProxy;
//      }

    /*******************************************************************************************************************
     *
     * Clears all flyweight caches (used for testing).
     *
     ******************************************************************************************************************/
    public static void flushFlyWeightCaches()
      {
//        logger.info("flushFlyWeightCaches()");
//        FLYWEIGHT_CACHE.clear();
      }
  }
