/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import javax.annotation.Nonnull;
import java.util.SortedSet;
import java.util.TreeSet;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.geocoding.GeoCoder;
import it.tidalwave.geo.geocoding.GeoCoderManager;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.GeoSchemaManager;
import static it.tidalwave.geo.geocoding.GeoCoderManager.GeoCoderManager;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=GeoSchemaManager.class)
public class ElmoGeoSchemaManager implements GeoSchemaManager
  {
    private final static String CLASS = ElmoGeoSchemaManager.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    @Override
    public synchronized GeoSchema findSchemaByName (@Nonnull final String name)
      throws NotFoundException
      {
        logger.fine("findSchemaByName(%s)", name);

        // FIXME: don't cache to manage different Repositories
        // You should instead query the repository first
        // TODO: how to keep a FlyWeight? Cache indexed by Repository?
        if (getSchemaNames().contains(name))
          {
            final GeoCoderManager geoCoderManager = Locator.find(GeoCoderManager);
            final GeoSchema schema = ElmoGeoSchemaFactory.findOrCreate(geoCoderManager.findGeoCoderByName(name));
            logger.finer(">>>> returning %s", schema);
            return schema;
          }

        throw new NotFoundException(name);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    @Override
    public SortedSet<String> getSchemaNames()
      {
        final TreeSet<String> result = new TreeSet<String>();

        for (final GeoCoder geoCoder : Locator.find(GeoCoderManager).findAllGeoCoders())
          {
            result.add(geoCoder.getName());
          }

        return result;
      }
  }
