/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.List;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoLocation.Finder;

/***********************************************************************************************************************
 *
 * FIXME: possibly get rid of this, and instead use GeoFinder. I think that only qualifiedName needs to be moved
 * and to integrate Wrapper. But Wrapper could be moved to the GeoFinderProvider.
 * 
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class FinderSupport extends it.tidalwave.util.spi.FinderSupport<GeoLocation, Finder> implements Finder
  {
    @CheckForNull
    protected String displayName;

    @CheckForNull
    protected String path;

    @CheckForNull
    protected Range range;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public FinderSupport (final @Nonnull String description)
      {
        super(description);
      }

    @Override
    public FinderSupport clone() 
      {
        final FinderSupport clone = (FinderSupport)super.clone();
        clone.displayName = this.displayName;
        clone.path = this.path;
        clone.range = this.range;
        
        return clone;
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder withDisplayName (final @Nonnull String displayName)
      {
        final FinderSupport clone = clone();
        clone.displayName = displayName;
        return clone;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder withQualifiedName (final @Nonnull String path)
      {
        final FinderSupport clone = clone();
        clone.path = path;
        return clone;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder closeTo (final @Nonnull Coordinate coordinate, final @Nonnegative double radius)
      {
        final FinderSupport clone = clone();
        clone.range = Range.on(coordinate).withRadius(radius);
        return clone;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public <X> it.tidalwave.util.Finder<X> ofType (final @Nonnull Class<X> type)
      {
        throw new UnsupportedOperationException("Not supported.");
      }
  }
