/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl.finder;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.Finder;
import it.tidalwave.semantic.Wrapper;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.finder.spi.GeoFinderProvider;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoCoderEntityProxyFactory;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoLocationFactory;
import it.tidalwave.geo.location.elmo.impl.FinderSupport;
import it.tidalwave.geo.location.elmo.impl.GeoCoderEntityProxy;
import static it.tidalwave.geo.GeoContainer.GeoContainer;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.GeoCoderEntity;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=GeoFinderProvider.class)
public class CoordinateFinder extends GeoFinderProvider<GeoLocation, Coordinate>
  {
    public CoordinateFinder()
      {
        super(Coordinate.class, GeoLocation.class);
      }

    @Override @Nonnull
    protected Finder<GeoLocation> find (final @Nonnull Coordinate coordinate,
                                        final @Nonnull Class<? extends GeoLocation> type,
                                        final @Nonnull Lookup parameters)
      {
        return new FinderSupport("")
          {
            @Override @Nonnull
            protected List<GeoLocation> computeResults()
              {
                final GeoSchema schema = FinderUtil.findGeoSchema(parameters);
                final List<GeoLocation> result = new ArrayList<GeoLocation>();

                for (final GeoCoderEntity gcEntity : coordinate.as(GeoContainer).find(GeoCoderEntity).results())
                  {
                    final GeoCoderEntityProxy gcEntityProxy =
                        ElmoGeoCoderEntityProxyFactory.findOrCreate(schema, gcEntity);
                    result.add(ElmoGeoLocationFactory.findOrCreate(schema, gcEntityProxy));
                  }

                return Wrapper.wrap(result);
              }
          };
      }
  }
