/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl.finder;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import it.tidalwave.geo.geocoding.GeoCoderManager;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.GeoSchemaManager;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class FinderUtil 
  {
    @CheckForNull
    private static GeoSchema defaultGeoSchema;

    @Nonnull
    public static synchronized GeoSchema findGeoSchema (final @Nonnull Lookup lookup)
      {
        GeoSchema geoSchema = lookup.lookup(GeoSchema.class);

        if (geoSchema == null)
          {
            synchronized (FinderUtil.class)
              {
                if (defaultGeoSchema == null)
                  {
                    try
                      {
                        final String name = Locator.find(GeoCoderManager.class).findDefaultGeoCoder().getName();
                        defaultGeoSchema = Locator.find(GeoSchemaManager.class).findSchemaByName(name);
                      }
                    catch (NotFoundException e)
                      {
                        throw new RuntimeException("Cannot find the default GeoCoder - is a provider installed?", e);
                      }
                  }

                geoSchema = defaultGeoSchema;
              }
          }

        return geoSchema;
      }
  }
