/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.semantic.Wrapper;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.geocoding.GeoCoderEntity.Type;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoPath;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.GeoSchemaManager;
import static it.tidalwave.geo.Units.*;
import static it.tidalwave.role.Displayable.Displayable;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * Tests the functions of the API starting from a scratch repository, populating it with data from the remote GeoCoder.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ElmoGeoLocationOnlineIntegrationTest extends ElmoGeoLocationIntegrationTestSupport
  {
    @Test
    public void shouldProperlyRetrieveLocationsCloseToAGivenPoint()
      throws Exception
      {
        final GeoSchemaManager schemaManager = Locator.find(GeoSchemaManager.class);
        final GeoSchema geoSchema = schemaManager.findSchemaByName("GeoNames");

        final List<? extends GeoLocation> results = geoSchema.findLocations().closeTo(new Coordinate(45, 11), km(5)).results();

        assertThat(results.size(), is(8));
        final Iterator<? extends GeoLocation> i = results.iterator();

        final GeoLocation location0 = i.next();
        assertGood(location0, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location0), is(true));
        assertThat(location0.as(Displayable).getDisplayName(), is("Quistello"));
        assertThat(location0.findCoordinate(geoSchema), is(new Coordinate(45.00558, 10.9823, 0)));
        assertThat(location0.findCode(geoSchema), is(""));
        assertThat(location0.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location1 = i.next();
        assertGood(location1, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location1), is(true));
        assertThat(location1.as(Displayable).getDisplayName(), is("Casone Sanguinetto"));
        assertThat(location1.findCoordinate(geoSchema), is(new Coordinate(45.01667, 11.0, 0)));
        assertThat(location1.findCode(geoSchema), is(""));
        assertThat(location1.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location2 = i.next();
        assertGood(location2, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location2), is(true));
        assertThat(location2.as(Displayable).getDisplayName(), is("San Rocco"));
        assertThat(location2.findCoordinate(geoSchema), is(new Coordinate(44.98333, 11.0, 0)));
        assertThat(location2.findCode(geoSchema), is(""));
        assertThat(location2.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location3 = i.next();
        assertGood(location3, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location3), is(true));
        assertThat(location3.as(Displayable).getDisplayName(), is("La Cavriana"));
        assertThat(location3.findCoordinate(geoSchema), is(new Coordinate(44.98333, 10.98333, 0)));
        assertThat(location3.findCode(geoSchema), is(""));
        assertThat(location3.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location4 = i.next();
        assertGood(location4, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location4), is(true));
        assertThat(location4.as(Displayable).getDisplayName(), is("Trivellano"));
        assertThat(location4.findCoordinate(geoSchema), is(new Coordinate(44.98333, 10.96667, 0)));
        assertThat(location4.findCode(geoSchema), is(""));
        assertThat(location4.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location5 = i.next();
        assertGood(location5, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location5), is(true));
        assertThat(location5.as(Displayable).getDisplayName(), is("Loghino Speranza"));
        assertThat(location5.findCoordinate(geoSchema), is(new Coordinate(44.98333, 11.03333, 0)));
        assertThat(location5.findCode(geoSchema), is(""));
        assertThat(location5.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location6 = i.next();
        assertGood(location6, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location6), is(true));
        assertThat(location6.as(Displayable).getDisplayName(), is("San Giacomo delle Segnate"));
        assertThat(location6.findCoordinate(geoSchema), is(new Coordinate(44.97258, 11.0331, 0)));
        assertThat(location6.findCode(geoSchema), is(""));
        assertThat(location6.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        final GeoLocation location7 = i.next();
        assertGood(location7, GeoLocation.class);
        assertThat(Wrapper.isWrapped(location7), is(true));
        assertThat(location7.as(Displayable).getDisplayName(), is("Cascina Fienili"));
        assertThat(location7.findCoordinate(geoSchema), is(new Coordinate(45.01667, 11.05, 0)));
        assertThat(location7.findCode(geoSchema), is(""));
        assertThat(location7.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        dump("itsearch-pre"); // TODO: compare exp file, it should only contain the 8 locations WITHOUT parents

        // TODO: compare exp file, it should only contain the 8 locations WITHOUT parents
        final GeoPath path0 = location0.findPath(geoSchema);
//        assertThat(path0.size(), is(6)); // FIXME: fails

        final Iterator<? extends GeoLocation> j = path0.iterator();
        final GeoLocation path0_0 = j.next();
        assertGood(path0_0, GeoLocation.class);
        assertThat(Wrapper.isWrapped(path0_0), is(true));
        assertThat(path0_0.as(Displayable).getDisplayName(), is("Earth"));
        assertThat(path0_0.findCoordinate(geoSchema), is(new Coordinate(0, 0, 0)));
        assertThat(path0_0.findCode(geoSchema), is(""));
//        assertThat(path0_0.findType(geoSchema), is((Object)Type.PLANET)); // FIXME

        final GeoLocation path0_1 = j.next();
        assertGood(path0_1, GeoLocation.class);
        assertThat(Wrapper.isWrapped(path0_1), is(true));
        assertThat(path0_1.as(Displayable).getDisplayName(), is("Europe"));
        assertThat(path0_1.findCoordinate(geoSchema), is(new Coordinate(48.69096, 9.14062, 0)));
        assertThat(path0_1.findCode(geoSchema), is("EU"));
        assertThat(path0_1.findType(geoSchema), is((Object)Type.CONTINENT));

        final GeoLocation path0_2 = j.next();
        assertGood(path0_2, GeoLocation.class);
        assertThat(Wrapper.isWrapped(path0_2), is(true));
        assertThat(path0_2.as(Displayable).getDisplayName(), is("Italy"));
        assertThat(path0_2.findCoordinate(geoSchema), is(new Coordinate(42.83333, 12.83333, 0)));
        assertThat(path0_2.findCode(geoSchema), is("IT"));
        assertThat(path0_2.findType(geoSchema), is((Object)Type.COUNTRY));

        final GeoLocation path0_3 = j.next();
        assertGood(path0_3, GeoLocation.class);
        assertThat(Wrapper.isWrapped(path0_3), is(true));
        assertThat(path0_3.as(Displayable).getDisplayName(), is("Lombardy"));
        assertThat(path0_3.findCoordinate(geoSchema), is(new Coordinate(45.66667, 9.5, 0)));
        assertThat(path0_3.findCode(geoSchema), is("09"));
        assertThat(path0_3.findType(geoSchema), is((Object)Type.ADMIN_DIVISION_1));

        final GeoLocation path0_4 = j.next();
        assertGood(path0_4, GeoLocation.class);
        assertThat(Wrapper.isWrapped(path0_4), is(true));
        assertThat(path0_4.as(Displayable).getDisplayName(), is("Province of Mantua"));
        assertThat(path0_4.findCoordinate(geoSchema), is(new Coordinate(45.16667, 10.78333, 0)));
        assertThat(path0_4.findCode(geoSchema), is("MN"));
        assertThat(path0_4.findType(geoSchema), is((Object)Type.ADMIN_DIVISION_2));

        final GeoLocation path0_5 = j.next();
        assertGood(path0_5, GeoLocation.class);
//        assertThat(path0_5, is(sameInstance(location0))); // FIXME: fails

//        assertPath(location1.findPath(geoSchema), path0, location1);
        assertPath(location2.findPath(geoSchema), path0, location2);
        assertPath(location3.findPath(geoSchema), path0, location3);
        assertPath(location4.findPath(geoSchema), path0, location4);
        assertPath(location5.findPath(geoSchema), path0, location5);
        assertPath(location6.findPath(geoSchema), path0, location6);
        assertPath(location7.findPath(geoSchema), path0, location7);

        dump("itsearch-final");
      }

    /*******************************************************************************************************************
     *
     * Not really a test, but the generator for the RDF dump that is used by the offline test.
     *
     ******************************************************************************************************************/
    @Test
    public void generateGlobalDump()
      throws Exception
      {
        final GeoSchemaManager schemaManager = Locator.find(GeoSchemaManager.class);
        final GeoSchema geoNamesSchema = schemaManager.findSchemaByName("GeoNames");
        assertNotNull(geoNamesSchema);

        for (final String path : Arrays.asList
             (
                 "/Europe/Italy/Liguria/Genoa/Recco",
                 "/Europe/France/Bourgogne/Yonne/Arrondissement d'Avallon/V\u00e9zelay/V\u00e9zelay",
                 "/North America/United States/California/Marin County/Sausalito"
             ))
          {
            final GeoPath ppp = geoNamesSchema.findLocations().withQualifiedName(path).result().findPath(geoNamesSchema);

            for (final GeoLocation l : ppp)
              {
                l.findChildren(geoNamesSchema).results();
              }
          }

        dump("all-final");
      }

    private static void assertPath (final @Nonnull GeoPath path,
                                    final @Nonnull GeoPath expectedPath,
                                    final @Nonnull GeoLocation tail)
      {
//        assertThat(path.size(), is(expectedPath.size())); FIXME: fails

        for (int i = 0; i < path.size() - 1; i++)
          {
//            assertThat(path.get(i), is(sameInstance(expectedPath.get(i)))); FIXME is Wrapper flyweight?
//            assertThat(Wrapper.unwrap(path.get(i)), is(sameInstance(Wrapper.unwrap(expectedPath.get(i))))); FIXME different instances
          }

//        assertThat(path.get(path.size() - 1), is(sameInstance(tail))); FIXME: fails
      }
  }