/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import org.openrdf.elmo.ElmoModule;
import org.openrdf.elmo.sesame.SesameManagerFactory;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.sail.memory.MemoryStore;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoCoderEntityProxyFactory;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoLocationFactory;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import it.tidalwave.semantic.IdFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TestSupport
  {
    private final static String CLASS = TestSupport.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    protected final static String GEO_CODER_NAME = "GeoNames";

    protected SailRepository sailRepository;

    private final ElmoModule elmoModule = new ElmoModule();

    private SesameManagerFactory smf;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(TestSupport.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @BeforeClass
    public static void prepareTestFacilities()
      {
        System.setProperty("it.tidalwave.geo.location.elmo.impl.FakeTimeStamp", "true");

        final IdFactory idFactory = Locator.find(IdFactory.class);

        if (!(idFactory instanceof TestIdFactory))
          {
            MockLookup.addBannedObject(idFactory);
            MockLookup.setInstances(TestIdFactory.getInstance());
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @AfterClass
    public static void restoreGeoCoder()
      {
        MockLookup.reset();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Before
    public void setupElmo()
      throws RepositoryException
      {
        logger.info("********** SETTING UP A SCRATCH REPOSITORY");
        sailRepository = new SailRepository(new MemoryStore());
        sailRepository.initialize();
        smf = new SesameManagerFactory(elmoModule, sailRepository);
        ElmoManagerThreadLocal.set(smf.createElmoManager());
//        ElmoManagerThreadLocal.get().getTransaction().begin();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Before
    public void flushFlyWeightCaches()
      {
        ElmoGeoLocationFactory.flushCaches();
        ElmoGeoCoderEntityProxyFactory.flushFlyWeightCaches();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @After
    public void tearDown()
      throws RepositoryException
      {
//        ElmoManagerThreadLocal.get().getTransaction().commit();
        logger.info("********** SHUTTING DOWN THE REPOSITORY");
//        sailRepository.shutDown(); FIXME: blocks as we're leaking connections!
      }
  }
