/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl.finder;

import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.geocoding.GeoCoderEntity.Type;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.GeoSchemaManager;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoLocationIntegrationTestSupport;
import it.tidalwave.geo.location.elmo.impl.TestSupport;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;
import java.util.Iterator;
import java.util.List;
import static it.tidalwave.role.Displayable.Displayable;
import static it.tidalwave.geo.Units.*;
import static it.tidalwave.geo.GeoContainer.GeoContainer;
import static it.tidalwave.geo.location.GeoLocation.GeoLocation;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class RangeFinderTest extends TestSupport
  {
//    @Test(expected=NullPointerException.class) should instead verify that the default is used
//    public void mustThrowExceptionWhenGeoSchemaIsNotSpecified() no more - the default is used
//      throws NotFoundException
//      {
//        new Coordinate(45, 11).as(GeoContainer).find(GeoLocation).result();
//      }

    @Test
    public void mustFindThreeLocationsAt45N11W2kmRange()
      throws NotFoundException
      {
        final GeoSchemaManager schemaManager = Locator.find(GeoSchemaManager.class);
        final GeoSchema geoSchema = schemaManager.findSchemaByName(GEO_CODER_NAME);
        assertThat(geoSchema, is(notNullValue()));

        final Range range = Range.on(new Coordinate(45, 11)).withRadius(km(2));
        final List<? extends GeoLocation> locations = range.as(GeoContainer).find(GeoLocation).with(geoSchema).results();
        assertThat(locations.size(), is(3));
        Iterator<? extends GeoLocation> i = locations.iterator();
        
        GeoLocation location = i.next();
        assertThat(location, is(notNullValue()));
        ElmoGeoLocationIntegrationTestSupport.assertGood(location, GeoLocation.class);
        assertThat(location.as(Displayable).getDisplayName(), is("Quistello"));
        assertThat(location.findCoordinate(geoSchema), is(new Coordinate(45.00558, 10.9823, 0)));
        assertThat(location.findCode(geoSchema), is(""));
        assertThat(location.findType(geoSchema), is((Object)Type.INHABITED_PLACE));

        location = i.next();
        assertThat(location, is(notNullValue()));
        ElmoGeoLocationIntegrationTestSupport.assertGood(location, GeoLocation.class);
        assertThat(location.as(Displayable).getDisplayName(), is("Casone Sanguinetto"));
//        assertThat(location.findCoordinate(geoSchema), is(new Coordinate(45.005576, 10.982295, 0))); FIXME
        assertThat(location.findCode(geoSchema), is(""));
        assertThat(location.findType(geoSchema), is((Object)Type.INHABITED_PLACE));
        
        location = i.next();
        assertThat(location, is(notNullValue()));
        ElmoGeoLocationIntegrationTestSupport.assertGood(location, GeoLocation.class);
        assertThat(location.as(Displayable).getDisplayName(), is("San Rocco"));
//        assertThat(location.findCoordinate(geoSchema), is(new Coordinate(45.005576, 10.982295, 0))); FIXME
        assertThat(location.findCode(geoSchema), is(""));
        assertThat(location.findType(geoSchema), is((Object)Type.INHABITED_PLACE));
      }
  }
