/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.node.impl;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import org.openide.nodes.Node;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.netbeans.nodes.role.RootNodeProvider;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=CapabilitiesProvider.class)
public class GeoLocationNodeCapabilityProvider implements CapabilitiesProvider<GeoSchema>
  {
    protected final Map<GeoSchema, GeoLocationNodeFactoryImpl> nodeFactoryMapByGeoSchema = new HashMap<GeoSchema, GeoLocationNodeFactoryImpl>();

    @Override
    @Nonnull
    public Class<GeoSchema> getManagedClass()
      {
        return GeoSchema.class;
      }

    @Override
    @Nonnull
    public Collection<? extends Object> createCapabilities (final @Nonnull GeoSchema geoSchema)
      {
        return Collections.singletonList(new RootNodeProvider()
          {
            @Override
            @Nonnull
            public Node getRootNode()
              {
                GeoLocationNodeFactoryImpl nodeFactory;

                synchronized (nodeFactoryMapByGeoSchema)
                  {
                    nodeFactory = nodeFactoryMapByGeoSchema.get(geoSchema);

                    if (nodeFactory == null)
                      {
                        nodeFactory = new GeoLocationNodeFactoryImpl(geoSchema);
                        nodeFactoryMapByGeoSchema.put(geoSchema, nodeFactory);
                      }
                  }

                return nodeFactory.getRootNode();
              }
          });
      }

    @Override
    @Nonnull
    public Collection<? extends Lookup> createLookups (final @Nonnull GeoSchema geoSchema)
      {
        return Collections.<Lookup>emptyList();
      }
  }
