/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.mock;

import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoPath;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.util.NotFoundException;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class MockGeoLocation implements GeoLocation
  {
    @Nonnull
    private MockGeoLocation parent;

    private final List<MockGeoLocation> children;
    
    private Coordinate coordinate;

    private Lookup lookup;

    private String displayName;

    public MockGeoLocation (final @Nonnull Coordinate coordinate,
                            final @Nonnull String displayName,
                            final @Nonnull MockGeoLocation ... children)
      {
        this.children = Arrays.asList(children);
        this.coordinate = coordinate;
        this.lookup = Lookup.EMPTY;
        this.displayName = displayName;

        for (final MockGeoLocation child : children)
          {
            child.parent = this;
          }
      }

    @Override
    public GeoLocation findParent (final @Nonnull GeoSchema schema)
      throws NotFoundException
      {
        if (parent == null)
          {
            throw new NotFoundException("No parent");
          }

        return parent;
      }

    @Override
    public Finder findChildren (final @Nonnull GeoSchema schema)
      {
        return new MockFinderSupport(children);
      }

    @Override
    public GeoPath findPath (final @Nonnull GeoSchema schema)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public Object findType (final @Nonnull GeoSchema geoSchema)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public String findCode (final @Nonnull GeoSchema geoSchema)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public Coordinate findCoordinate (final @Nonnull GeoSchema geoSchema)
      throws NotFoundException
      {
        return coordinate;
      }

    @Override
    public void setCoordinate (final @Nonnull Coordinate coordinate)
      {
        this.coordinate = coordinate;
      }

    @Override
    public void setCode (final @Nonnull String code)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public Builder createChild()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public Lookup getLookup()
      {
        return lookup;
      }

    @Override
    public <T> T as(Class<T> clazz)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public <T> T as(Class<T> clazz, NotFoundBehaviour<T> notFoundBehaviour)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }
