/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.node.impl;

import java.util.Collection;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.nodes.role.RootNodeProvider;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.Mockito.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoLocationNodeCapabilityProviderTest extends TestSupport
  {
    private GeoLocationNodeCapabilityProvider fixture;

    @Before
    public void setupFixture()
      {
        fixture = new GeoLocationNodeCapabilityProvider();
      }

    @Test
    public void shouldBeRegisteredInTheDefaultLookup()
      {
        Collection<? extends CapabilitiesProvider> capabilities = Lookup.getDefault().lookupAll(CapabilitiesProvider.class);
        assertThat(capabilities, is(notNullValue()));
        assertThat(capabilities.size(), is(1));
        assertThat(capabilities.iterator().next(), is(instanceOf(GeoLocationNodeCapabilityProvider.class)));
      }

    @Test
    public void shouldDeclareToManageGeoSchema()
      {
        assertEquals(GeoSchema.class, fixture.getManagedClass());
//        assertThat(fixture.getManagedClass(), is(eq(GeoSchema.class))); FIXME
      }

    @Test
    public void shouldCreateARootNodeProviderWhichReturnsANodeWrappingTheRootGeoLocation()
      throws NotFoundException
      {
        final GeoLocation rootGeoLocation = mock(GeoLocation.class);
        when(geoSchema.getRoot()).thenReturn(rootGeoLocation);
        
        final Collection<? extends Object> capabilities = fixture.createCapabilities(geoSchema);
        
        assertThat(capabilities, is(notNullValue()));
        assertThat(capabilities.size(), is(1));
        assertThat(capabilities.iterator().next(), instanceOf(RootNodeProvider.class));
        final RootNodeProvider rootNodeProvider = (RootNodeProvider)capabilities.iterator().next();
        final Node rootNode = rootNodeProvider.getRootNode();
        assertThat(rootNode, is(notNullValue()));
        final Lookup lookup = rootNode.getLookup();
        assertThat(lookup, is(notNullValue()));
        final GeoLocation lookedUpGeoLocation = lookup.lookup(GeoLocation.class);
        assertThat(lookedUpGeoLocation, is(sameInstance(rootGeoLocation)));
      }

    @Test
    public void shouldCreateNoLookups()
      {
        final Collection<? extends Lookup> lookups = fixture.createLookups(geoSchema);
        
        assertThat(lookups, is(notNullValue()));
        assertThat(lookups.isEmpty(), is(true));
      }

    @Test
    public void shouldCreateASingleNodeFactoryForEachSchema()
      throws NotFoundException
      {
        final GeoSchema geoSchema2 = mock(GeoSchema.class);
        final GeoLocation rootGeoLocation = mock(GeoLocation.class);
        when(geoSchema2.getRoot()).thenReturn(rootGeoLocation);

        assertThat(fixture.nodeFactoryMapByGeoSchema.size(), is(0));

        for (int i = 0; i < 10; i++)
          {
            ((RootNodeProvider)fixture.createCapabilities(geoSchema).iterator().next()).getRootNode();
          }

        assertThat(fixture.nodeFactoryMapByGeoSchema.size(), is(1));

        for (int i = 0; i < 10; i++)
          {
            ((RootNodeProvider)fixture.createCapabilities(geoSchema2).iterator().next()).getRootNode();
          }

        assertThat(fixture.nodeFactoryMapByGeoSchema.size(), is(2));
      }
  }