/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.node.impl;

import java.awt.Image;
import org.openide.util.Lookup;
import org.openide.nodes.Node;
import it.tidalwave.geo.location.GeoLocation;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoLocationNodeFactoryImplTest extends TestSupport
  {
    private GeoLocationNodeFactoryImpl fixture;
    private GeoLocation rootGeoLocation;

    @Before
    public void setUp()
      {
        rootGeoLocation = mock(GeoLocation.class);
        fixture = new GeoLocationNodeFactoryImpl(geoSchema);
        doReturn(rootGeoLocation).when(geoSchema).getRoot();
      }

    @Test
    public void shouldReturnARootNodeThatWrapsTheRootGeoLocation()
      {
        final Node rootNode = fixture.getRootNode();
        assertThat(rootNode, is(not(nullValue())));
        final Lookup lookup = rootNode.getLookup();
        assertThat(lookup, is(not(nullValue())));
        final GeoLocation lookedUpGeoLocation = lookup.lookup(GeoLocation.class);
        assertThat(lookedUpGeoLocation, sameInstance(rootGeoLocation));
      }

    @Test
    public void shouldAlwaysReturnTheSameNonNullIcon()
      {
        final Image icon1 = fixture.findIcon(mock(Node.class));
        final Image icon2 = fixture.findIcon(mock(Node.class));

        assertThat(icon1, is(notNullValue()));
        assertThat(icon2, is(notNullValue()));
        assertThat(icon1, is(sameInstance(icon2)));
      }
  }