/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.geocoding.impl.finder;

import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import java.util.Iterator;
import java.util.List;
import org.junit.BeforeClass;
import static it.tidalwave.role.Displayable.Displayable;
import static it.tidalwave.geo.GeoContainer.GeoContainer;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.GeoCoderEntity;
import org.junit.Test;
import static it.tidalwave.geo.Units.*;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class RangeIntegrationTest
  {
    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(RangeIntegrationTest.class);
      }

    @Test
    public void mustFindPlacesNear45N11W()
      throws NotFoundException
      {
        final Range range = Range.on(new Coordinate(45, 11)).withRadius(km(5));
        final List<? extends GeoCoderEntity> entities = range.as(GeoContainer).find(GeoCoderEntity).results();
        assertThat(entities, is(notNullValue()));
        assertThat(entities.size(), is(8));
        final Iterator<? extends GeoCoderEntity> i = entities.iterator();

        GeoCoderEntity entity = i.next();
        assertThat(entity.getId(), is("3169656"));
        assertThat(entity.as(Displayable).getDisplayName(), is("Quistello"));
        assertThat(entity.getCoordinate(), is (new Coordinate(45.00558, 10.9823, 0)));

        entity = i.next();
        assertThat(entity.getId(), is("3167774"));
        assertThat(entity.as(Displayable).getDisplayName(), is("San Rocco"));
        assertThat(entity.getCoordinate(), is (new Coordinate(44.98333, 11.0, 0)));

        entity = i.next();
        assertThat(entity.getId(), is("3179816"));
        assertThat(entity.as(Displayable).getDisplayName(), is("Casone Sanguinetto"));
        assertThat(entity.getCoordinate(), is (new Coordinate(45.01667, 11.0, 0)));

        // TODO: further 5 assertions
      }
  }