/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.node.flagsoftheworld.impl;

import javax.annotation.Nonnull;
import java.awt.Image;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import it.tidalwave.swing.IconResizer;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.netbeans.nodes.GenericDecoratorNode;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
// FIXME: if you manage in getting an abstraction for providing the country code, rename to GeoNodeWithFlag (not FlagS)
@edu.umd.cs.findbugs.annotations.SuppressWarnings("HE_INHERITS_EQUALS_USE_HASHCODE, EQ_DOESNT_OVERRIDE_EQUALS")
public final class GeoEntityNodeWithFlags extends GenericDecoratorNode
  {
    @Nonnull
    private final Node delegate;

    public GeoEntityNodeWithFlags (final @Nonnull Node delegate)
      {
        super(delegate);
        this.delegate = delegate;
      }

    @Override
    @Nonnull 
    public Image getOpenedIcon (final int type) 
      {
        return getIcon(type);
      }

    @Override
    @Nonnull
    public Image getIcon (final int type)
      {
        final GeoCoderEntity geoCoderEntity = delegate.getLookup().lookup(GeoCoderEntity.class);

        Image image = null;

        if (geoCoderEntity != null) 
          {
            final String code = geoCoderEntity.getCode();

            if (geoCoderEntity.getType() == GeoCoderEntity.Type.COUNTRY)
              {
                image = ImageUtilities.loadImage("it/tidalwave/geo/node/flagsoftheworld/impl/icons/" + code + ".png");

                if (image != null)
                  {
                    image = IconResizer.scaledImage(image, 16);
                  }
              }
            else
              {
                final Node parentNode = getParentNode();

                if (parentNode != null)
                  {
                    image = parentNode.getIcon(type);
                  }
              }

          }

        return (image != null) ? image : delegate.getIcon(type);
      }
  }
