/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import org.jdesktop.observablecollections.ObservableList;
import it.tidalwave.geo.Coordinate;

/***********************************************************************************************************************
 *
 * This class allows to add or remove objects ('features') from/to a {@link GeoViewPresentation}.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ThreadSafe
public interface FeatureManager
  {
    /*******************************************************************************************************************
     *
     * Adds a feature to the given {@code Coordinate}.
     *
     * @param   feature     the feature to add
     * @param   coordinate  the coordinate to place the feature at
     *
     ******************************************************************************************************************/
    public void addFeature (@Nonnull Object feature, @Nonnull Coordinate coordinate);

    /*******************************************************************************************************************
     *
     * Adds a feature. The feature must have some implicit {@code Coordinate} according to some registration done
     * earlier (for instance, a {@link Track} or a {@link Sequence}.
     *
     * @param   feature                    the feature to add
     * @throws  IllegalArgumentException   if can't guess the coordinate for the feature
     *
     ******************************************************************************************************************/
    public void addFeature (@Nonnull Object feature);

    /*******************************************************************************************************************
     *
     * Removes a feature.
     *
     * @param   feature                    the feature to remove
     *
     ******************************************************************************************************************/
    public void removeFeature (@Nonnull Object feature);

    /*******************************************************************************************************************
     *
     * Removes all the features.
     *
     ******************************************************************************************************************/
    public void removeAllFeatures();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ObservableList<Object> getFeatures();
  }
