/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeListener;
import org.jdesktop.observablecollections.ObservableList;
import it.tidalwave.geo.viewer.spi.GeoViewProvider;

/***********************************************************************************************************************
 *
 * This service allows to control which geographic provider is providing data for the geographic view.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface GeoViewProviderManager
  {
    public static final String PROP_SELECTED_GEO_VIEW_PROVIDER = "selectedGeoViewProvider";
    public static final String PROP_GEO_VIEW_PROVIDERS = "geoViewProviders";

    /*******************************************************************************************************************
     *
     * Sets the current {@link GeoViewProvider}. This is a bound property.
     *
     * @param  the new {@code GeoViewProvider}
     *
     ******************************************************************************************************************/
    public void setSelectedGeoViewProvider (@Nonnull GeoViewProvider selectedGeoViewProvider);

    /*******************************************************************************************************************
     *
     * Gets the current {@link GeoViewProvider}.
     *
     * @param  the new {@code GeoViewProvider}
     *
     ******************************************************************************************************************/
    @Nonnull
    public GeoViewProvider getSelectedGeoViewProvider();

    /*******************************************************************************************************************
     *
     * Returns the list of available {@link GeoViewProvider}s. A bound list is returned.
     *
     * @return  the list of available {@code GeoViewProvider}s
     *
     ******************************************************************************************************************/
    @Nonnull
    public ObservableList<GeoViewProvider> getGeoViewProviders();

    /*******************************************************************************************************************
     *
     * Registers a {@link PropertyChangeListener}.
     * 
     * @param  listener   the listener
     *
     ******************************************************************************************************************/
    public void addPropertyChangeListener (@Nonnull PropertyChangeListener listener);

    /*******************************************************************************************************************
     *
     * Unregisters a {@link PropertyChangeListener}.
     *
     * @param  listener   the listener
     *
     ******************************************************************************************************************/
    public void removePropertyChangeListener (@Nonnull PropertyChangeListener listener);
  }
