/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.impl.role;

import javax.annotation.Nonnull;
import javax.annotation.PostConstruct;
import javax.inject.Inject;
import org.netbeans.platformx.eventbus.api.EventBus;
import org.netbeans.platformx.eventbus.api.EventBusListener;
import it.tidalwave.netbeans.windows.role.PresentationLocker;
import it.tidalwave.netbeans.windows.role.TopComponentHelper;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.geocoding.GeoCoderEntitySet;
import it.tidalwave.geo.viewer.FeatureManager;
import it.tidalwave.geo.viewer.GeoViewManager;
import it.tidalwave.geo.viewer.ZoomLevel;
import javax.inject.Provider;

/***********************************************************************************************************************
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class EventBusGeoEntitySetListener
  {
    @Inject
    private TopComponentHelper topComponentHelper;

    @Inject
    private Provider<PresentationLocker> presentationLocker;

    @Inject
    private Provider<GeoViewManager> mapManager;

    @Inject
    private Provider<FeatureManager> featureManager;

    @Inject
    private EventBus eventBus;
    
    private final EventBusListener<GeoCoderEntitySet> geoEntitySetListener = new EventBusListener<GeoCoderEntitySet>()
      {
        @Override
        public void notify (final @Nonnull GeoCoderEntitySet geoEntitySet)
          {
            if (geoEntitySet != null)
              {
                topComponentHelper.openAndRequestActive(); // FIXME: move to a separate role
                presentationLocker.get().lock("Loading data...");

                try
                  {
                    // FIXME: split the role that adds features from the one that fits the view
                    for (final GeoCoderEntity geoEntity : geoEntitySet)
                      {
                        featureManager.get().addFeature(geoEntity, geoEntity.getCoordinate());
                      }

                    mapManager.get().fitView(geoEntitySet.getSector(), ZoomLevel.REGION);
                  }
                finally
                  {
                    presentationLocker.get().unlock();
                  }
              }
          }
      };

    @PostConstruct
    @SuppressWarnings("PMD.UnusedPrivateMethod") @edu.umd.cs.findbugs.annotations.SuppressWarnings("UPM_UNCALLED_PRIVATE_METHOD")
    private void initialize()
      {
        eventBus.subscribe(GeoCoderEntitySet.class, geoEntitySetListener);
      }
  }
