/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.role;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.util.ArrayList;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.jdesktop.observablecollections.ObservableCollections;
import org.jdesktop.observablecollections.ObservableList;
import it.tidalwave.geo.viewer.spi.GeoViewProvider;
import it.tidalwave.netbeans.windows.role.PresentationLocker;
import it.tidalwave.geo.viewer.impl.GeoRendererPanel;

/***********************************************************************************************************************
 *
 * The base class for the UI of {@link it.tidalwave.geo.viewer.GeoViewerPresentation GeoViewerPresentation}. Any
 * implementation, including custom ones, should subclass from it.
 * 
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class GeoViewerUI extends JPanel implements PresentationLocker, GeoViewProvider.CoordinateTracker
  {
    public static final String PROP_SELECTED_GEO_VIEW_PROVIDER = "selectedGeoViewProvider";
    public static final String PROP_SELECTED_MODE = "selectedMode";

    private final ObservableList<GeoViewProvider> geoViewProviders =
            ObservableCollections.observableList(new ArrayList<GeoViewProvider>());

    @Inject
    private GeoRendererPanel geoRendererPanel;

    @CheckForNull
    private GeoViewProvider selectedGeoViewProvider;

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void setSelectedGeoViewProvider (final @Nullable GeoViewProvider selectedGeoViewProvider)
      {
        final GeoViewProvider oldSelectedGeoViewProvider = this.selectedGeoViewProvider;
        this.selectedGeoViewProvider = selectedGeoViewProvider;
        firePropertyChange(PROP_SELECTED_GEO_VIEW_PROVIDER, oldSelectedGeoViewProvider, selectedGeoViewProvider);
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    public final GeoViewProvider getSelectedGeoViewProvider()
      {
        return selectedGeoViewProvider;
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public final ObservableList<GeoViewProvider> getGeoViewProviders()
      {
        return geoViewProviders;
      }

    /*******************************************************************************************************************
     *
     * Returns the component which renderers the geographic data. Since the component is injected, this method can't
     * be called by the constructor; instead must be called from an initializer annotated with {@link PostConstruct}
     * (or from a method executed after the initializer).
     *
     * @return  the renderer component
     *
     ******************************************************************************************************************/
    @Nonnull
    protected final JComponent getRendererComponent()
      {
        if (geoRendererPanel == null)
          {
            throw new AssertionError("This method must be called from a method annotated with @PostConstruct");
          }

        return geoRendererPanel;
      }
  }
