/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.impl;

import java.lang.reflect.InvocationTargetException;
import java.awt.EventQueue;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.geo.viewer.spi.GeoViewProviderSupport;
import it.tidalwave.geo.viewer.spi.GeoViewProvider;
import it.tidalwave.geo.viewer.impl.spi.VoidGeoViewProvider;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import it.tidalwave.netbeans.windows.role.PresentationLocker;
import javax.annotation.Nonnull;
import javax.inject.Provider;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultGeoViewProviderManagerTest
  {
    private DefaultGeoViewProviderManager fixture;
    private PresentationLocker presentationLocker;
    private Lookup lookup;
    private GeoViewProvider provider1;

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(TestLoggerSetup.class);
      }

    @Before
    public void setUp() 
      {
        fixture = new DefaultGeoViewProviderManager();
        presentationLocker = mock(PresentationLocker.class);
        lookup = Lookups.fixed();
        fixture.lookup = lookup;
        fixture.presentationLocker = new Provider<PresentationLocker>()
          {
            @Override @Nonnull
            public PresentationLocker get()
              {
                return presentationLocker;
              }
          };

        provider1 = mock(GeoViewProviderSupport.class);
        when(provider1.isReady()).thenReturn(true);
      }

    @Test
    public void testPostConstructWithNoProviders()
      throws Exception
      {
        fixture.lookup = Lookup.EMPTY;
        callPostConstruct();
        assertThat(fixture.geoViewProviders.size(), is(0));
        assertThat(fixture.selectedGeoViewProvider, is(instanceOf(VoidGeoViewProvider.class)));
      }

    @Test
    public void testPostConstructWithValidProvider()
      throws Exception
      { 
        fixture.lookup = Lookups.fixed(provider1);
        callPostConstruct();
        assertThat(fixture.geoViewProviders.size(), is(1));
        assertThat(fixture.selectedGeoViewProvider, is(sameInstance(provider1)));
      }

    private void callPostConstruct()
      throws InterruptedException, InvocationTargetException
      {
        EventQueue.invokeAndWait(new Runnable()
          {
            @Override
            public void run()
              {
                fixture.initialize();
              }
          });
      }
  }