/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.impl;

import java.awt.EventQueue;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import it.tidalwave.netbeans.windows.role.TopComponentHelper;
import it.tidalwave.geo.viewer.FeatureManager;
import it.tidalwave.geo.viewer.GeoViewProviderManager;
import it.tidalwave.geo.viewer.GeoViewManager;
import it.tidalwave.geo.viewer.role.CacheLocationProvider;
import it.tidalwave.geo.viewer.role.GeoViewerUI;
import it.tidalwave.geo.viewer.role.InitialLocationProvider;
import it.tidalwave.geo.viewer.role.ZeroZeroInitialLocationProvider;
import it.tidalwave.geo.viewer.impl.role.DefaultGeoViewerUI;
import it.tidalwave.geo.viewer.impl.role.EventBusGeoEntitySetListener;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.hamcrest.core.IsInstanceOf;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoViewerTopComponentTest
  {
    private GeoViewerTopComponent fixture;
    private static EnhancedWindowManager windowManager;

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(TestLoggerSetup.class);
      }

    @Before
    public void setUp()
      throws Exception
      {
        windowManager = mock(EnhancedWindowManager.class);
        MockLookup.setInstances(windowManager);
        fixture = GeoViewerTopComponent.findInstance();
        EventQueue.invokeAndWait(new Runnable()
          {
            @Override
            public void run()
              {
                fixture.installRoles();
              }
          });
      }

    @Test
    public void testGeoViewProviderManagerIsPresent()
      {
        final GeoViewProviderManager role = fixture.getLookup().lookup(GeoViewProviderManager.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(DefaultGeoViewProviderManager.class));
      }

    @Test
    public void testGeoViewManagerIsPresent()
      {
        final GeoViewManager role = fixture.getLookup().lookup(GeoViewManager.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(DefaultGeoViewManager.class));
      }

    @Test
    public void testEventBusGeoEntitySetListenerIsPresent()
      {
        final EventBusGeoEntitySetListener role = fixture.getLookup().lookup(EventBusGeoEntitySetListener.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(EventBusGeoEntitySetListener.class));
      }

    @Test
    public void testTopComponentHelperIsPresent()
      {
        final TopComponentHelper role = fixture.getLookup().lookup(TopComponentHelper.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(TopComponentHelper.class));
      }

    @Test
    public void testInitialLocationProviderIsPresent()
      {
        final InitialLocationProvider role = fixture.getLookup().lookup(InitialLocationProvider.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(ZeroZeroInitialLocationProvider.class));
      }

    @Test
    public void testFeatureManagerIsPresent()
      {
        final FeatureManager role = fixture.getLookup().lookup(FeatureManager.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(FeatureManager.class));
      }

    @Test
    public void testCacheSettingsIsPresent()
      {
        final CacheLocationProvider role = fixture.getLookup().lookup(CacheLocationProvider.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(CacheLocationProvider.class));
      }

    @Test
    public void testGeoViewerUIIsPresent()
      {
        final GeoViewerUI role = fixture.getLookup().lookup(GeoViewerUI.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(DefaultGeoViewerUI.class));
      }

    @Test
    public void testGeoRendererPanelIsPresent()
      {
        final GeoRendererPanel role = fixture.getLookup().lookup(GeoRendererPanel.class);
        assertThat(role, is(not(nullValue())));
        assertThat(role, IsInstanceOf.instanceOf(GeoRendererPanel.class));
      }
  }