/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.jxmapviewer;

import javax.inject.Inject;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.dialog.DialogRunner;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.GeoTag;
import it.tidalwave.geo.viewer.GeoViewManager;
import it.tidalwave.netbeans.visual.NodeScene;
import it.tidalwave.netbeans.visual.NodeScene.DragEvent;
import org.openide.nodes.Node;
import static it.tidalwave.netbeans.visual.NodeScene.DragBehaviour.Confirmation.*;

/***********************************************************************************************************************
 *
 * A {@link NodeScene.DragBehaviour} to apply to Widgets representing a pin - dragging them means to change the
 * related {@link Coordinate}.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoTaggingDragBehaviour implements NodeScene.DragBehaviour
  {
    private static final String CLASS = GeoTaggingDragBehaviour.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Inject
    private GeoViewManager geoViewManager;
    
    @CheckForNull
    private Coordinate previousCoordinates;

    @CheckForNull
    private GeoTag adjustingGeoTag;

    @CheckForNull
    private Node draggedNode;

    @Override
    public void nodeDragStarted (final @Nonnull DragEvent event)
      {
        draggedNode = event.getNode();
        adjustingGeoTag = event.getNode().getLookup().lookup(GeoTag.class);

        if (adjustingGeoTag != null)
          {
            previousCoordinates = adjustingGeoTag.getCoordinate();
          }
      }

    @Override
    public void nodeDragged (final @Nonnull DragEvent event)
      {
        if (adjustingGeoTag != null)
          {
            final Coordinate coordinates = geoViewManager.pointToCoordinate(event.getLocation());

            try
              {
                adjustingGeoTag.setCoordinate(coordinates);
              }
            catch (Exception e)
              {
                logger.throwing(CLASS, "nodeDragged()", e);
              }
          }
      }

    @Override
    public void nodeDragFinished (final @Nonnull DragEvent event,
                                  final @Nonnull Confirmation confirmation)
      {
        if ((confirmation == CANCELLED) && (adjustingGeoTag != null) && (previousCoordinates != null)) // undo changes
          {
            try
              {
                adjustingGeoTag.setCoordinate(previousCoordinates);
              }
            catch (Exception e)
              {
                logger.throwing(CLASS, "nodeDragFinished()", e);
              }
          }

        adjustingGeoTag = null;
        previousCoordinates = null;
        draggedNode = null;
      }

    @Override @Nonnull
    public Confirmation confirmDrag (final @Nonnull DragEvent event)
      {
        final String string1 = "Confirm new position?";
        final String string2 = "Geotags will be updated";
        final String title   = "Confirm";

        return DialogRunner.showYesNoConfirmation(string1, string2, title) ? CONFIRMED : CANCELLED;
      }

    @CheckForNull
    public Node getDraggedNode()
      {
        return draggedNode;
      }
  }