/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.jxmapviewer;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.painter.Painter;
import org.jdesktop.swingx.mapviewer.GeoPosition;
import it.tidalwave.geo.viewer.spi.feature.FeatureManagerSpi;
import it.tidalwave.geo.viewer.spi.feature.FeaturesRenderer;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class JXMapViewerFeaturePainter implements Painter<JXMapViewer>
  {
    private static final String CLASS = JXMapViewerFeaturePainter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Inject
    private FeaturesRenderer featureRenderer;
    
    @Inject
    private FeatureManagerSpi featureManagerSpi;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void paint (final Graphics2D g, final JXMapViewer mapViewer, final int width, final int height) 
      {
        logger.fine("paint(" + g + ", " + mapViewer + ", " + width + ", " + height + ")");
        final Rectangle viewportBounds = mapViewer.getViewportBounds();

        for (final Object object : new ArrayList<Object>(featureManagerSpi.getFeatures()))
          {
            final GeoPosition geoPosition = CoordinateConverter.toGeoPosition(featureManagerSpi.findCoordinate(object));
            final Point2D point = mapViewer.getTileFactory().geoToPixel(geoPosition, mapViewer.getZoom());

//            if (waypointManager.isRunning())
//              {
//                final long deltaTime = waypointManager.getTimeThreshold() - waypoint.getTimestamp();

//                if (deltaTime < 0)
//                  {
//                    continue;
//                  }

/*                if (deltaTime <= TRAIL_TIME)
                  {
                    int alpha = (int)Math.min(127, (127 * deltaTime) / TRAIL_TIME);
                    color = new Color(128 + alpha, 0, 255, 255 - alpha);
                  }*/
//              }

            if (viewportBounds.contains(point))
              {
                final int x = (int)(point.getX() - viewportBounds.getX());
                final int y = (int)(point.getY() - viewportBounds.getY());
                featureRenderer.paintFeature(g, object, x, y, true, null);
              }
          }
      }
  }
