/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.jxmapviewer.widget;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import javax.annotation.Nonnull;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import it.tidalwave.netbeans.visual.widget.ConnectingWidget;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GeoConnectingWidget extends ConnectingWidget implements Widget.Dependency
  {
    private final Color selectedColor = new Color(255, 255, 255);

    private final Color color = new Color(200, 200, 200);

    private final @Nonnull Widget sourceWidget;

    /***************************************************************************
     *
     *
     **************************************************************************/
    public GeoConnectingWidget (final @Nonnull Scene scene,
                                final @Nonnull Widget sourceWidget, 
                                final @Nonnull Widget targetWidget)
      {
        super(scene, sourceWidget, targetWidget);
        this.sourceWidget = sourceWidget;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    protected void paintWidget()
      {
        final Graphics2D g = getGraphics();
        g.setColor(sourceWidget.getState().isSelected() ? selectedColor : color);
        final Point p1 = getFirstControlPoint();
        final Point p2 = getLastControlPoint();
        final int[] x = new int[3];
        final int[] y = new int[3];
        x[0] = p2.x;
        y[0] = p2.y;
        float dx = p2.x - p1.x;
        float dy = p2.y - p1.y;
        final float l = (float)Math.sqrt(dx * dx + dy * dy);
        dx /= l;
        dy /= l;
        final int r = 6; // base size of the wedge
        final int e = -6; // how long it protrudes into the Widget
        x[1] = p1.x + Math.round(dx * e + dy * r);
        y[1] = p1.y + Math.round(dy * e - dx * r);
        x[2] = p1.x + Math.round(dx * e - dy * r);
        y[2] = p1.y + Math.round(dy * e + dx * r);
        final Polygon polygon = new Polygon(x, y, 3);
        g.fillPolygon(polygon);
      }

    @Override @Nonnull
    public String toString()
      {
        return String.format("GeoConnectingWidget@%x[%s]", System.identityHashCode(this), sourceWidget);
      }

    @Override
    public void revalidateDependency()
      {
          System.err.println("revalidate");
      }
  }
