/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.jxmapviewer.widget;

import javax.annotation.Nonnull;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.widget.LabelWidget;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import it.tidalwave.geo.viewer.spi.jxmapviewer.widget.OffsetWidgetDragBehaviour;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class OffsetWidget extends LabelWidget implements Widget.Dependency 
  {
    @Nonnull
    private final Lookup lookup;

    private final Point offset = new Point(0, -128);

    @Nonnull
    private Widget anchorWidget;

    @Nonnull
    private Widget containedWidget;

    public OffsetWidget (final @Nonnull Scene scene, 
                         final @Nonnull Widget containedWidget,
                         final @Nonnull Widget anchorWidget)
      {
        super(scene);

        this.lookup = new ProxyLookup(containedWidget.getLookup(), Lookups.fixed(new OffsetWidgetDragBehaviour()));
        this.containedWidget = containedWidget;
        this.anchorWidget = anchorWidget;

        setOpaque(true);
        setBorder(org.netbeans.api.visual.border.BorderFactory.createEmptyBorder());
        setLayout(LayoutFactory.createHorizontalFlowLayout());
        addChild(containedWidget);
      }

    public void translateOffset (final int dx, final int dy)
      {
        offset.translate(dx, dy);
      }

    @Override
    public void revalidateDependency()
      {
        int dx = 0;
        int dy = 0;

        Rectangle clientArea = getClientArea();

        if (clientArea != null)
          {
            dx -= clientArea.width / 2;
            dy -= clientArea.height / 2;
          }

        final Insets insets = getBorder().getInsets();

        if (insets != null)
          {
            dx += insets.left;
            dy += insets.top;
          }

        // Use preferredLocation, since location is only updated *after* validation. Can be null at initialization.
        final Point anchorLocation = anchorWidget.getPreferredLocation();

        if (anchorLocation != null)
          {
            setPreferredLocation(new Point(anchorLocation.x + offset.x + dx, anchorLocation.y + offset.y + dy));
          }
      }

    @Override @Nonnull
    public Lookup getLookup()
      {
        return lookup;
      }

    @Override @Nonnull
    public String toString()
      {
        return String.format("OffsetWidget@%x[%s]", System.identityHashCode(this), containedWidget);
      }
  }
