/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.microsoftvirtualearth;

import java.net.MalformedURLException;
import java.net.URL;
import it.tidalwave.role.Displayable;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class MicrosoftVirtualEarthTileMapProviderDescriptorTest extends MicrosoftVirtualEarthTileProviderDescriptorTest
  {
    @Before
    public void setupFixture()
      {
        fixture = new MicrosoftVirtualEarthTileMapProviderDescriptor();
      }

    @Test
    public void testSpecificProperties()
      {
        assertThat(fixture.getDataCacheName(), is("MVE/r"));
        assertThat(fixture.as(Displayable.class).getDisplayName(), is("Microsoft Virtual Earth (map)"));
      }

    @Test
    public void testURLs()
      throws MalformedURLException
      {
        assertThat(fixture.getTileUrl(1, 0, 0), is(new URL("http://r0.ortho.tiles.virtualearth.net/tiles/r0.png?g=1")));

        assertThat(fixture.getTileUrl(2, 0, 0), is(new URL("http://r0.ortho.tiles.virtualearth.net/tiles/r00.png?g=1")));
        assertThat(fixture.getTileUrl(2, 0, 1), is(new URL("http://r2.ortho.tiles.virtualearth.net/tiles/r02.png?g=1")));
        assertThat(fixture.getTileUrl(2, 1, 0), is(new URL("http://r1.ortho.tiles.virtualearth.net/tiles/r01.png?g=1")));
        assertThat(fixture.getTileUrl(2, 1, 1), is(new URL("http://r3.ortho.tiles.virtualearth.net/tiles/r03.png?g=1")));

        assertThat(fixture.getTileUrl(10,   3,   6), is(new URL("http://r1.ortho.tiles.virtualearth.net/tiles/r0000000231.png?g=1")));
        assertThat(fixture.getTileUrl(10,  50, 123), is(new URL("http://r2.ortho.tiles.virtualearth.net/tiles/r0002332032.png?g=1")));
        assertThat(fixture.getTileUrl(10, 257, 493), is(new URL("http://r3.ortho.tiles.virtualearth.net/tiles/r0322202203.png?g=1")));
        assertThat(fixture.getTileUrl(10, 904, 712), is(new URL("http://r0.ortho.tiles.virtualearth.net/tiles/r3132003000.png?g=1")));
      }
  }