/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.impl;

import javax.annotation.Nonnull;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Sector;

public class MercatorSector extends Sector
  {
    @Nonnull
    private final double minLatPercent;

    @Nonnull
    private final double maxLatPercent;

    public MercatorSector (final double minLatPercent,
                           final double maxLatPercent,
                           final @Nonnull Angle minLongitude,
                           final @Nonnull Angle maxLongitude)
      {
        super(gudermannian(minLatPercent), gudermannian(maxLatPercent), minLongitude, maxLongitude);
        this.minLatPercent = minLatPercent;
        this.maxLatPercent = maxLatPercent;
      }

    @Nonnull
    public static MercatorSector fromDegrees (final double minLatPercent,
                                              final double maxLatPercent,
                                              final double minLongitude,
                                              final double maxLongitude)
      {
        return new MercatorSector(minLatPercent,
                                  maxLatPercent,
                                  Angle.fromDegrees(minLongitude),
                                  Angle.fromDegrees(maxLongitude));
      }

    @Nonnull
    public static MercatorSector fromSector (final @Nonnull Sector sector)
      {
        return new MercatorSector(gudermannianInverse(sector.getMinLatitude()),
                                  gudermannianInverse(sector.getMaxLatitude()),
                                  new Angle(sector.getMinLongitude()),
                                  new Angle(sector.getMaxLongitude()));
      }

    public static double gudermannianInverse (final @Nonnull Angle latitude)
      {
        //see http://en.wikipedia.org/wiki/Mercator_projection
        return Math.log(Math.tan(Math.PI / 4.0 + latitude.radians / 2.0)) / Math.PI;
      }

    @Nonnull
    public static Angle gudermannian (final double percent)
      {
        //see http://en.wikipedia.org/wiki/Mercator_projection
        return Angle.fromRadians(Math.atan(Math.sinh(percent * Math.PI)));
      }

    public double getMinLatPercent()
      {
        return minLatPercent;
      }

    public double getMaxLatPercent()
      {
        return maxLatPercent;
      }
  }
