/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.impl;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Iterator;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.LatLon;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.viewer.spi.feature.FeatureManagerSpi;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class WaypointToLatLonAdapter implements Iterable<LatLon>
  {
    @Nonnull
    private final Collection<Object> features;

    @Nonnull
    private final FeatureManagerSpi featureManagerSpi;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public WaypointToLatLonAdapter (final @Nonnull Collection<Object> features,
                                    final @Nonnull FeatureManagerSpi featureManagerSpi)
      {
        this.features = features;
        this.featureManagerSpi = featureManagerSpi;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public Iterator<LatLon> iterator() 
      {
        return new Iterator<LatLon>()
          {
            private final Iterator<Object> iterator = features.iterator();
    
            @Override
            public boolean hasNext() 
              {
                return iterator.hasNext();
              }

            @Override
            @Nonnull
            public LatLon next() 
              {
                final Coordinate coordinate = featureManagerSpi.findCoordinate(iterator.next());
                return new LatLon(Angle.fromDegreesLatitude(coordinate.getLatitude()),
                                  Angle.fromDegreesLongitude(coordinate.getLongitude()));
              }

            @Override
            public void remove() 
              {
                iterator.remove();
              }
          };
      }
  }
