/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.impl.layer;

import java.util.ArrayList;
import java.awt.Color;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.RenderableLayer;
import gov.nasa.worldwind.render.Polyline;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GridLayer extends RenderableLayer
  {
    private static final Color COLOR = new Color(0, 255, 255, 64);
    private static final Angle MIN = Angle.fromDegrees(-85);
    private static final Angle MAX = Angle.fromDegrees(+85);
    private static final int NUM_EDGE_INTERVALS = 36;
    private static final double H = 500; // 10e3;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public GridLayer()
      {
        setName("Grid");
        
        // Generate meridians
        final ArrayList<Position> positions = new ArrayList<Position>(3);
        
        for (double lon = -180; lon < 180; lon += 10) 
          {
            final Angle longitude = Angle.fromDegrees(lon);
            positions.clear();
            positions.add(new Position(MIN, longitude, H));
            positions.add(new Position(Angle.ZERO, longitude, H));
            positions.add(new Position(MAX, longitude, H));
            final Polyline polyline = new Polyline(positions);
//            polyline.setNumEdgeIntervals(NUM_EDGE_INTERVALS);
            polyline.setNumSubsegments(NUM_EDGE_INTERVALS);
            polyline.setColor(COLOR);
            addRenderable(polyline);
          }
        
        // Generate parallels
        for (double lat = -85; lat < 90; lat += 10) 
          {
            final Angle latitude = Angle.fromDegrees(lat);
            positions.clear();
            positions.add(new Position(latitude, Angle.NEG180, H));
            positions.add(new Position(latitude, Angle.ZERO, H));
            positions.add(new Position(latitude, Angle.POS180, H));
            final Polyline polyline = new Polyline(positions);
            polyline.setNumSubsegments(NUM_EDGE_INTERVALS);
//            polyline.setNumEdgeIntervals(NUM_EDGE_INTERVALS);
            polyline.setColor(COLOR);
            addRenderable(polyline);
          }
      }
  }
