/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.impl.layer;

import javax.annotation.Nonnull;
import javax.media.opengl.GL;
import gov.nasa.worldwind.render.DrawContext;
import gov.nasa.worldwind.render.Renderable;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class RenderableSupport implements Renderable
  {
    protected int glListId = -1;
      
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void render (final @Nonnull DrawContext dc)
      {
        if (glListId == -1)
          {
            makeGList(dc);
          }
        
        final GL gl = dc.getGL();
        gl.glPushAttrib(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT
                        | GL.GL_POLYGON_BIT | GL.GL_TEXTURE_BIT | GL.GL_ENABLE_BIT
                        | GL.GL_CURRENT_BIT);
        gl.glPushMatrix();
            
        try
          {
            gl.glDisable(GL.GL_TEXTURE_2D);     
            gl.glDisable(GL.GL_DEPTH_TEST);      
            // Set far clipping far enought - is this the right way to do it ?
            gl.glMatrixMode(GL.GL_PROJECTION);
            gl.glLoadIdentity();
            double limits[] = computeFrustumLimits(dc);
            dc.getGLU().gluPerspective(dc.getView().getFieldOfView().degrees,
                            dc.getView().getViewport().getWidth() / dc.getView().getViewport().getHeight(),
                            limits[0], limits[1]);
            gl.glCallList(glListId);
          }
         
        finally
          {
            gl.glPopMatrix();
            gl.glPopAttrib();
          }
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    abstract protected void createGList (@Nonnull DrawContext dc, @Nonnull GL gl);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    abstract protected double[] computeFrustumLimits (@Nonnull DrawContext dc);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void makeGList (final @Nonnull DrawContext dc)
      {
        GL gl = dc.getGL();
        glListId = gl.glGenLists(1);
        // FIXME: what about garbaging the old one?
        gl.glNewList(glListId, GL.GL_COMPILE);
        createGList(dc, gl);
        gl.glEndList();      
      }
  }

