/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.impl.layer;

import java.util.ArrayList;
import java.util.List;
import javax.media.opengl.GL;
import it.tidalwave.geo.Coordinate;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Vec4;
import gov.nasa.worldwind.render.DrawContext;
import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class WaypointRenderable extends RenderableSupport
  {
    class Wpt
      {
        Coordinate coordinates;
        Object object;
      }
    
    private final List<Wpt> waypointList = new ArrayList<Wpt>();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void addObject (final @Nonnull Coordinate coordinates, final @Nonnull Object object)
      {
        final Wpt waypoint = new Wpt();
        waypoint.coordinates = coordinates;
        waypoint.object = object;
        waypointList.add(waypoint);
        glListId = -1;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clear()
      {
        waypointList.clear();
        glListId = -1;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected double[] computeFrustumLimits (final @Nonnull DrawContext dc)
      {
        final double r = dc.getGlobe().getEquatorialRadius();
        final Vec4 center = dc.getGlobe().getCenter();
        final double distanceFromOrigin = dc.getView().getEyePoint().distanceTo3(center);
        final double near = 1000; // ditanceFromOrigine - r * 2;
        final double far = distanceFromOrigin;
        return new double[]{ near, far };
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void createGList (final @Nonnull DrawContext dc, final @Nonnull GL gl)
      {
        final double sphereRadius = dc.getGlobe().getEquatorialRadius();

        float Vdec = 0.9f;
        float hhh = 3300000; // 3400000;
        
/*        for (double lat = -80; lat <= 80; lat += 5)
          {
            gl.glBegin(GL.GL_LINE_STRIP);
            gl.glColor3f((float)Vdec, (float)0, (float)0); 
            
            for (double lon = -180; lon < 180; lon += 1)
              {
                final Point pos = dc.getGlobe().computePointFromPosition(Angle.fromDegreesLatitude(lat), Angle.fromDegreesLongitude(lon), hhh);
                gl.glVertex3d(pos.getX(), pos.getY(), pos.getZ());
              }
            
            gl.glEnd();
          }

        for (double lon = -180; lon <= 180; lon += 5)
          {
            gl.glBegin(GL.GL_LINE_STRIP);
            gl.glColor3f((float)Vdec, (float)0, (float)0); 
            
            for (double lat = -80; lat <= 80; lat += 1)
              {
                final Point pos = dc.getGlobe().computePointFromPosition(Angle.fromDegreesLatitude(lat), Angle.fromDegreesLongitude(lon), hhh);
                gl.glVertex3d(pos.getX(), pos.getY(), pos.getZ());
              }
            
            gl.glEnd();
          }*/

        for (final Wpt yyy : waypointList)
          {
            final Vec4 pos = dc.getGlobe().computePointFromPosition(Angle.fromDegreesLatitude(yyy.coordinates.getLatitude()),
                                                                    Angle.fromDegreesLongitude(yyy.coordinates.getLongitude()), 
                                                                    hhh);
            gl.glBegin(GL.GL_POINTS);
            gl.glColor3f(Vdec, 0, 0); // grayscale
        gl.glPointSize(Vdec);
            gl.glVertex3d(pos.getX(), pos.getY(), pos.getZ());
            gl.glEnd();
          }
      }
  }
