/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.spi;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import gov.nasa.worldwind.WorldWind;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.layers.TextureTile;
import gov.nasa.worldwind.util.Level;
import gov.nasa.worldwind.util.TileKey;
import org.openide.util.Parameters;
import it.tidalwave.geo.viewer.worldwindprovider.impl.MercatorSector;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class MercatorTextureTile extends TextureTile
  {
    private MercatorSector mercatorSector;

    public MercatorTextureTile (final @Nonnull MercatorSector mercatorSector,
                                final @Nonnull Level level,
                                final @Nonnegative int row,
                                final @Nonnegative int column)
      {
        super(mercatorSector, level, row, column);
        this.mercatorSector = mercatorSector;
      }

    @Override @Nonnull
    public MercatorTextureTile[] createSubTiles (final @Nonnull Level nextLevel)
      {
        Parameters.notNull("nextLevel", nextLevel);

        final double minLatitude = mercatorSector.getMinLatPercent();
        final double maxLatitude = mercatorSector.getMaxLatPercent();
        final double midLatitude = minLatitude + (maxLatitude - minLatitude) / 2.0;

        final Angle minLongitude = getSector().getMinLongitude();
        final Angle maxLongitude = getSector().getMaxLongitude();
        final Angle midLongitude = Angle.midAngle(minLongitude, maxLongitude);

        final String nextLevelCacheName = nextLevel.getCacheName();
        final int nextLevelNum = nextLevel.getLevelNumber();
        final int row    = 2 * getRow();
        final int column = 2 * getColumn();

        final TileKey[] key = new TileKey[4];
        key[0] = new TileKey(nextLevelNum, row,     column,     nextLevelCacheName); // sw
        key[1] = new TileKey(nextLevelNum, row,     column + 1, nextLevelCacheName); // se
        key[2] = new TileKey(nextLevelNum, row + 1, column,     nextLevelCacheName); // nw
        key[3] = new TileKey(nextLevelNum, row + 1, column + 1, nextLevelCacheName); // ne

        final MercatorTextureTile[] subTiles = new MercatorTextureTile[4];
        subTiles[0] = findOrCreateTile(minLatitude, midLatitude, minLongitude, midLongitude, nextLevel, key[0]);
        subTiles[1] = findOrCreateTile(minLatitude, midLatitude, midLongitude, maxLongitude, nextLevel, key[1]);
        subTiles[2] = findOrCreateTile(midLatitude, maxLatitude, minLongitude, midLongitude, nextLevel, key[2]);
        subTiles[3] = findOrCreateTile(midLatitude, maxLatitude, midLongitude, maxLongitude, nextLevel, key[3]);

        return subTiles;
      }

    @Nonnull
    private MercatorTextureTile findOrCreateTile (final double minLatPercent,
                                                  final double maxLatPercent,
                                                  final @Nonnull Angle minLongitude,
                                                  final @Nonnull Angle maxLongitude,
                                                  final @Nonnull Level nextLevel,
                                                  final @Nonnull TileKey key)
      {
        MercatorTextureTile textureTile = 
                (MercatorTextureTile)WorldWind.getMemoryCache(MercatorTextureTile.class.getName()).getObject(key);

        if (textureTile == null)
          {
            final MercatorSector sector = new MercatorSector(minLatPercent, maxLatPercent, minLongitude, maxLongitude);
            textureTile = new MercatorTextureTile(sector, nextLevel, key.getRow(), key.getColumn());
          }

        return textureTile;
      }
  }
