/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.worldwindprovider.spi;

import it.tidalwave.geo.viewer.worldwindprovider.impl.MercatorSector;
import javax.annotation.Nonnull;
import java.net.MalformedURLException;
import java.net.URL;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.avlist.AVList;
import gov.nasa.worldwind.avlist.AVListImpl;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.util.LevelSet;
import gov.nasa.worldwind.util.Tile;
import gov.nasa.worldwind.util.TileUrlBuilder;
import it.tidalwave.geo.viewer.spi.TileProviderDescriptor;
import static it.tidalwave.role.Displayable.Displayable;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TiledImageLayer extends BasicMercatorTiledImageLayer
  {
    private static class UrlBuilderAdapter implements TileUrlBuilder
      {
        @Nonnull
        private final TileProviderDescriptor descriptor;

        public UrlBuilderAdapter (final @Nonnull TileProviderDescriptor descriptor)
          {
            this.descriptor = descriptor;
          }

        @Override @Nonnull
        public URL getURL (final @Nonnull Tile tile, final @Nonnull String string)
          throws MalformedURLException
          {
            final int levelNumber = tile.getLevelNumber() + 2 + 1 /* FIXME */;
//            row = row ^ -1; // Needed because of a different origin
            return descriptor.getTileUrl(levelNumber, tile.getColumn(), (1 << levelNumber) - tile.getRow() - 1);
          }
      }

    @Nonnull
    private final TileProviderDescriptor descriptor;

    public TiledImageLayer (final @Nonnull TileProviderDescriptor descriptor)
      {
        super(new LevelSet(makeParams(descriptor)), descriptor.as(Displayable).getDisplayName());
        this.descriptor = descriptor;
        setSplitScale(1.3); // FIXME: what's this?
      }

    @Nonnull
    public /* FIXME */ static AVList makeParams (final @Nonnull TileProviderDescriptor descriptor)
      {
        final AVList params = new AVListImpl();

        params.setValue(AVKey.TILE_WIDTH, descriptor.getTileSize().width);
        params.setValue(AVKey.TILE_HEIGHT, descriptor.getTileSize().height);
        params.setValue(AVKey.DATA_CACHE_NAME, descriptor.getDataCacheName());
        params.setValue(AVKey.SERVICE, "http://xxx/xxx");
        params.setValue(AVKey.DATASET_NAME, "x");
        params.setValue(AVKey.FORMAT_SUFFIX, ".xxx");
        params.setValue(AVKey.NUM_LEVELS, descriptor.getLevelCount());
        params.setValue(AVKey.NUM_EMPTY_LEVELS, 0);
        final int n = 4;
        params.setValue(AVKey.LEVEL_ZERO_TILE_DELTA, new LatLon(Angle.fromDegrees(90.0 / n), Angle.fromDegrees(180.0 / n)));
        params.setValue(AVKey.SECTOR, new MercatorSector(-1.0, +1.0, Angle.NEG180, Angle.POS180));
        params.setValue(AVKey.TILE_URL_BUILDER, new UrlBuilderAdapter(descriptor));

        return params;
      }

    @Override @Nonnull
    public String toString()
      {
        return descriptor.as(Displayable).getDisplayName();
      }
  }
