/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * Copyright (C) 2007-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://forceten.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/forceten-src
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.openstreetmap;

import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;
import it.tidalwave.role.Displayable;
import it.tidalwave.geo.viewer.spi.TileProviderDescriptor.HorizontalLayout;
import it.tidalwave.geo.viewer.spi.TileProviderDescriptor.VerticalLayout;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class OpenStreetMapTileProviderDescriptorTest
  {
    protected OpenStreetMapTileProviderDescriptor fixture;

    @Before
    public void setupFixture()
      {
        fixture = new OpenStreetMapTileProviderDescriptor();
      }

    @Test
    public void testGeneralProperties()
      {
        assertThat(fixture.getTileSize(), is(new Dimension(256, 256)));
        assertThat(fixture.getMinZoomLevel(), is(3));
        assertThat(fixture.getMaxZoomLevel(), is(17));
        assertThat(fixture.getTopZoomLevel(), is(17));
        assertThat(fixture.getInitialZoomLevel(), is(13));
        assertThat(fixture.getHorizontalLayout(), is(HorizontalLayout.LEFT_TO_RIGHT));
        assertThat(fixture.getVerticalLayout(), is(VerticalLayout.BOTTOM_TO_TOP));
        assertThat(fixture.getDataCacheName(), is("OSM"));
        assertThat(fixture.as(Displayable.class).getDisplayName(), is("OpenStreetMap"));
      }

    @Test
    public void testURLs()
      throws MalformedURLException
      {
        assertThat(fixture.getTileUrl(1, 0, 0), is(new URL("http://tile.openstreetmap.org/1/0/0.png")));

        assertThat(fixture.getTileUrl(2, 0, 0), is(new URL("http://tile.openstreetmap.org/2/0/0.png")));
        assertThat(fixture.getTileUrl(2, 0, 1), is(new URL("http://tile.openstreetmap.org/2/0/1.png")));
        assertThat(fixture.getTileUrl(2, 1, 0), is(new URL("http://tile.openstreetmap.org/2/1/0.png")));
        assertThat(fixture.getTileUrl(2, 1, 1), is(new URL("http://tile.openstreetmap.org/2/1/1.png")));

        assertThat(fixture.getTileUrl(10,   3,   6), is(new URL("http://tile.openstreetmap.org/10/3/6.png")));
        assertThat(fixture.getTileUrl(10,  50, 123), is(new URL("http://tile.openstreetmap.org/10/50/123.png")));
        assertThat(fixture.getTileUrl(10, 257, 493), is(new URL("http://tile.openstreetmap.org/10/257/493.png")));
        assertThat(fixture.getTileUrl(10, 904, 712), is(new URL("http://tile.openstreetmap.org/10/904/712.png")));
      }
  }